;==========================================================================================
; Programm mit Subroutinen fr den EEPROM
;
; Visitec GmbH Dezember 2005 by N.Fritsch   	     
; 
; EEPROM.inc
;==========================================================================================
;
; DEFINIERTE ROUTINEN SIND:
;
;	RD_EEPROMSP:	Dient zum Auslesen von bis zu 10 Bytes aus den entsprechenden 
;			Adressbereich des EEPROM. Vor Aufruf dieser Routine sind 
;			folgende Parameter zu setzen:
;							- Auszufhrende Operation (EEPROM_INSTR)
;							- Adresse fr Daten (EEPROM_ADR)
;							- Datenanzahl (EEPROM_COUNT)
;							 
;	WR_EEPROMSP:	Dient zum Schreiben von bis zu 10 Bytes in den entsprechenden 
;			Adressbereich des EEPROM. Vor Aufruf dieser Routine sind 
;			folgende Parameter zu setzen:
;							- Auszufhrende Operation (EEPROM_INSTR)
;							- Adresse fr Daten (EEPROM_ADR)
;							- Datenanzahl (EEPROM_COUNT)
;							- entsprechenden Datenbytes (EEPROM_DATA0 ...EEPROM_DATA9)
;
;	RD_EEPROMSR:	Dient zum Auslesen des Status Registers des EEPROM. Ergebnis ist in
;			EEPROM_DATA0 zu finden.
;		
;	WR_EEPROMSR:	Dient zum Beschreiben des Status Registers des EEPROM. Vor Aufruf ist 
;			das zu schreibende Datenbyte in EEPROM_DATA0 zu legen.						
;				
;	WR_ENABLE:	Zum Setzen des Write Enable Bits
;
;	WR_DISABLE:	Zum Setzen des Write Disable Bits	
;
;
; Verwendung im Hauptprogramm: 			include "EEPROM.inc"
;
; Bemerkungen:		Maximal 10 Bytes sind in einer Operation beschreibbar bzw. auslesbar. 
;			Vor dieser Routine ist der INCLUDE File: include "SPI_INTERFACE.inc"	
;			(Laden der Subroutinen fr SPI INTERFACE) im Pogramm zu laden.		
; 
; Speicherbereich($E8F-$EFA)
; Lnge ($6C = 107) Speicherwrter
;---------------------------------------------------------------------------------------------
;			s e r i e l l e  K o m u n i k a t i o n   m i t   E E P R O M
;=============================================================================================

;-Steuerleitungen des SX fr EEPROM-
EEPROM_CS	equ	RA.6				;CS des EEPROMs

;Konstante fr Trinamics Kommunikation
EEPROM_WREN	equ	%00000110			;Write Enable
EEPROM_WRDI	equ	%00000100			;Write Disable

EEPROM_RDSR	equ	%00000101			;Status-Register lesen
EEPROM_WRSR	equ	%00000001			;Status-Register schreiben

EEPROM_RDUP	equ	%00000111			;Lesen der oberen Hlfte des Speichers 
EEPROM_RDFL	equ	%00000011			;Lesen der unteren Hlfte des Speichers 

EEPROM_WRUP	equ	%00000110 			;Schreiben der oberen Hlfte des Speichers 
EEPROM_WRFL	equ	%00000010 			;Schreiben der unteren Hlfte des Speichers 
	
;Variablendefinition fr EEPROM
EEPROM_INSTR	equ	$70				;Zur Auswahl der Operation
EEPROM_ADR	equ	$71				;Speicheradresse
EEPROM_COUNT	equ	$72				;Anzahl der Datenbytes

EEPROM_DATA0	equ	$73				;Daten aus Speicheradresse
EEPROM_DATA1	equ	$74				;Daten aus Speicheradresse
EEPROM_DATA2	equ	$75				;Daten aus Speicheradresse
EEPROM_DATA3	equ	$76				;Daten aus Speicheradresse
EEPROM_DATA4	equ	$77				;Daten aus Speicheradresse
EEPROM_DATA5	equ	$78				;Daten aus Speicheradresse
EEPROM_DATA6	equ	$79				;Daten aus Speicheradresse
EEPROM_DATA7	equ	$7A				;Daten aus Speicheradresse
EEPROM_DATA8	equ	$7B				;Daten aus Speicheradresse
;-------------------------------------------------------------------------------------------------------------

;=============================================================================================================

;-Macros-
;----------- Macro zur Auswahl des EEPROM Bausteins ----------
EEPROM_Select	macro							
		clrb EEPROM_CS
endm
;-------------------------------------------------------------


;---------- Macro zur Abwhlen des EEPROM Bausteins ----------
EEPROM_UnSelect	macro							
		setb EEPROM_CS
endm
;-------------------------------------------------------------


;===============================================================================================================
;-Speicheradresse fr Programmcode-
org $E8F 


;-Subroutinen-
;---------- Routine zum Lesen des EEPROM Speichers -----------
RD_EEPROMSP	
		mov FSR,#EEPROM_INSTR			;Instruction Register laden
		
		EEPROM_Select				;EEPROM auswhlen

		page $C00
		     call sendbyte			;Read - Befehl senden
		page $E00

		mov FSR,#EEPROM_ADR			;Adresse des Registers mit der EEPROM - Register - Adresse laden
	
		page $C00
		     call sendbyte			;Adresse senden
		page $E00

		mov HILFSREG,#EEPROM_DATA0		;Datenbyte0 laden
		
REPEAT_RDSP	
		mov FSR,HILFSREG			;Ensprechendes Register ins FSR laden

		page $C00
		     call getbyte			;Byte vom EEPROM empfangen
		page $E00

		inc FSR					;nchstes Byte adressieren

		mov HILFSREG,FSR			;Zwischenspeichern des FSR Registers

		mov FSR,#EEPROM_COUNT			;Anzahl der Daten die ausgelesen werden sollen
		decsz INDF				;und eins weniger zu bearbeiten
		      jmp REPEAT_RDSP	

		EEPROM_UnSelect				;EEPROM abwhlen
ret
;-------------------------------------------------------------


;------- Routine zum Beschreiben des EEPROM Speichers --------
WR_EEPROMSP	
		mov FSR,#EEPROM_INSTR			;Instruction Register laden
		
		EEPROM_Select				;EEPROM auswhlen

		page $C00
		     call sendbyte			;Read - Befehl senden
		page $E00

		mov FSR,#EEPROM_ADR			;Adresse des Registers mit der EEPROM - Register - Adresse laden
	
		page $C00
		     call sendbyte			;Adresse senden
		page $E00

		mov HILFSREG,#EEPROM_DATA0		;Datenbyte0 laden
		
REPEAT_WRSP	
		mov FSR,HILFSREG			;Ensprechendes Register ins FSR laden

		page $C00
		     call sendbyte			;Byte vom EEPROM empfangen
		page $E00

		inc FSR					;nchstes Byte adressieren

		mov HILFSREG,FSR			;Zwischenspeichern des FSR Registers

		mov FSR,#EEPROM_COUNT			;Anzahl der Daten die ausgelesen werden sollen
		decsz INDF				;und eins weniger zu bearbeiten
		      jmp REPEAT_WRSP	

		EEPROM_UnSelect				;EEPROM abwhlen
ret
;-------------------------------------------------------------


;------- Routine zum Lesen des EEPROM Status-Registers -------
RD_EEPROMSR
		mov FSR,#EEPROM_INSTR			;Instruction Register anwhlen
		mov INDF,#EEPROM_RDSR			;Code des Read laden

		EEPROM_Select				;EEPROM auswhlen
		
		page $C00
		     call sendbyte
		page $E00

		mov FSR,#EEPROM_DATA0			;Datenbyte0 anwhlen

		page $C00
		     call getbyte			;Byte vom EEPROM empfangen
		page $E00

		EEPROM_UnSelect				;EEPROM abwhlen		
ret
;-------------------------------------------------------------


;------- Routine zum Beschreiben des EEPROM Status-Registers -------
WR_EEPROMSR
		mov FSR,#EEPROM_INSTR			;Instruction Register anwhlen
		mov INDF,#EEPROM_WRSR			;Code des Write laden

		EEPROM_Select				;EEPROM auswhlen
		
		page $C00
		     call sendbyte
		page $E00

		mov FSR,#EEPROM_DATA0			;Datenbyte0 anwhlen

		page $C00
		     call sendbyte			;Byte an EEPROM senden
		page $E00

		EEPROM_UnSelect				;EEPROM abwhlen		
ret
;-------------------------------------------------------------


;---------- Zum Setzen des Write Enable Signals ------------
WR_ENABLE
		mov FSR,#EEPROM_INSTR			;Instruction Register anwhlen
		mov INDF,#EEPROM_WREN			;Code des Write Enable

		EEPROM_Select				;EEPROM auswhlen
		
		page $C00
		     call sendbyte
		page $E00

		EEPROM_UnSelect				;EEPROM abwhlen	
ret
;-------------------------------------------------------------


;---------- Zum Setzen des Write Disable Signals ------------
WR_DISABLE
		mov FSR,#EEPROM_INSTR			;Instruction Register anwhlen
		mov INDF,#EEPROM_WRDI			;Code des Write Disable

		EEPROM_Select				;EEPROM auswhlen
		
		page $C00
		     call sendbyte
		page $E00

		EEPROM_UnSelect				;EEPROM abwhlen	
ret
;-------------------------------------------------------------


WaitToWriteEeprom
WTWEeprom
		call RD_EEPROMSR
		jb INDF.0,WTWEeprom
ret
;-------------------------------------

;=================================================================
