;==========================================================================================
; Programm mit Subroutinen fr die Encoder
;
; Visitec GmbH Februar 2008 by N.Fritsch   	     
; 
; Encoder.inc
;==========================================================================================
;
; DEFINIERTE ROUTINEN SIND:
;
;	Tri_Init:	Macro zur Initialisierung der Trinamics. Diese werden definiert zurckgesetzt.
;
;	Tri_Data_IN:	Datenport wird von SX Seite als Ausgang gelegt ==> Daten in Trinamics
;
;	Tri_Data_OUT:	Datenport wird von SX Seite als Eingang gelegt ==> Daten aus Trinamics
;
;	Para_Write:	Routine zur Auswahl des entsprechendne Trinamic, in den geschrieben werden soll.
;			Notwendige Daten die vor Aufruf dieser Routine gesetzt sein mssen sind:
;						- Auswahl des Trinamic (Trinamic_Selc) (0=1; 1=2; 2= beide)
;						- Adresse fr Register des Trinamics (Trinamic_Adr)
;						- Daten zum Schreiben in ausgewhlte Adresse (Trinamic_Data)
;
;	Para_Read:	Routine zur Auswahl des entsprechendne Trinamic, aus den gelesen werden soll.
;			Notwendige Daten die vor Aufruf dieser Routine gesetzt sein mssen sind:
;						- Auswahl des Trinamic (Trinamic_Selc) (0=1; 1=2; 2= beide)
;						- Adresse fr Register des Trinamics (Trinamic_Adr)
;	
;	StatusMotor:	Bit0	Trinamic 0 ist ausgewhlt (0)/ Trinamic 1 ist ausgewhlt (1)
;			Bit1	beide Trinamics sind ausgewhlt (1)
;			Bit2    ---
;			Bit3    ---
;			Bit4    --- 
;			Bit5    ---
;			Bit6    Motor od. Motoren sind aktiv (1)
;			Bit7    NotStop soll ausgefhrt werden (1)
;
; Verwendung im Hauptprogramm: 	include "Trinamics.inc"
;
; Bemerkungen:	----		
;
; Speicherbereich($800-$9CB)
; Lnge ($1CC = 460) Speicherwrter
;-------------------------------------------------------------------------------------------
;		s e r i e l l e   K o m u n i k a t i o n   m i t   E n c o d e r
;===========================================================================================
;Fr Biss Interface:
;-------------------
Enc0_CS		equ RA.3	;SX.OUT_A3 	(Enc1_CS)
Enc0_EOT	equ RA.4	;SX.IN_A4  	(Enc1_EOT)
Enc0_NER	equ RA.5	;SX.IN/OUT_A5	(Enc1_NER)

Enc1_CS		equ RB.1	;SX.OUT_B1 	(Enc2_CS)
Enc1_EOT	equ RB.2	;SX.IN_B2  	(Enc2_EOT)
Enc1_NER	equ RB.3	;SX.IN/OUT_B3	(Enc2_NER)

Enc_GetSens	equ RB.6	;SX.OUT_B6 	(Enc_GetSens)
Enc_NRes	equ RB.7	;SX.OUT_B7  	(Enc_NRes)


;Konstantendefinition fuer Biss Interface
Ins_BissWriteData	equ	%00000010
Ins_BissReadData	equ	%00000011
Ins_BissReadStatus	equ	%00000101
Ins_BissWriteInstr	equ	%00000111

;Variablendefinition fuer Biss Interface
MB3z_ICSelc	equ	$50
MB3z_Befehl	equ	$51
MB3z_Adr	equ	$52
MB3z_DatW0	equ	$53
MB3z_DatW1	equ	$54
MB3z_DatW2	equ	$55
MB3z_DatW3	equ	$56
MB3z_DatW4	equ	$57
MB3z_DatW5	equ	$58
MB3z_DatCount	equ	$59

IndexPosD0	equ	$5A
IndexPosD1	equ	$5B
IndexPosD2	equ	$5C

SensPosD0	equ	$5D
SensPosD1	equ	$5E

Direction	equ	$60
StatusSensorDat equ	$61
;Bit 7  BREAK gesetzt
;Bit 6	Daten lesen
;Bit 5  Fehler Abbruch GetSens ERROR 
;Bit 4	Fehler Abbruch EOT ERROR
;Bit 3	Fehler Abbruch nERR ERROR
;Bit 2  Zhler fr EOT = 0
;Bit 1  Zhler fr EOT = 0
;Bit 0  Zhler fr EOT = 0
;-------------------------------------------------------------------------------------------


;===========================================================================================
;-Speicheradresse fr Programmcode-
org $800	

;Definition von Sprungadressen fr Subroutinen
L_subpos	jmp subpos
L_addpos	jmp addpos
L_CheckSensMsg  jmp CheckSensMsg

;-Subroutine
EncBissData
		clrb SCK_SX			;ALE Clock zurcksetzen
		mov FSR,#MB3z_ICSelc		;Auswahl des entsprechenden BISS Interface Bausteins
		csne INDF,#$00			;berprfen ob BISS Interface 0 beschrieben werden soll
		    clrb Enc0_CS			;aktivieren BISS Interface 0
		
		csne INDF,#$01			;berprfen ob BISS Interface 1 beschrieben werden soll
		    clrb Enc1_CS			;aktivieren BISS Interface 1
		
		csne INDF,#$02			;berprfen ob beide BISS Interface beschrieben werden sollen	
    		    clrb Enc0_CS			;aktivieren BISS Interface 0
		csne INDF,#$02
    		    clrb Enc1_CS			;aktivieren BISS Interface 1

		mov FSR,#MB3z_Befehl		;Senden des Befehls
		page $C00
		     call sendbyte
		page $800

		;Laden der Anzahl zu bearbeitender Datan
		mov FSR,#MB3z_DatCount
		mov HILFSREG,INDF
		
		;Ueberpruefung ob Befehl Read Status
		mov FSR,#MB3z_Befehl
		cje INDF,#Ins_BissReadStatus,Read_BissDat	
		cje INDF,#Ins_BissWriteInstr,Write_BissDat

		mov FSR,#MB3z_Adr
		page $C00
		     call sendbyte
		page $800

		;Ueberpruefung ob Read Befehl
		mov FSR,#MB3z_Befehl
		cje INDF,#Ins_BissReadData,Read_BissDat	
		;---
Write_BissDat
		;Schreiben von Daten
		mov FSR,#MB3z_DatW0
Repeat_WR		
		page $C00
		     call sendbyte
		page $800

		INC FSR				;Zeiger auf eine Adresse weiter stellen 	
		DEC HILFSREG
		sz 
		  jmp Repeat_WR	

		jmp EndBissData
		;---
Read_BissDat
		mov FSR,#MB3z_DatW0
Repeat_RD		
		page $C00
		     call getbyte
		page $800

		INC FSR				;Zeiger auf eine Adresse weiter stellen 	
		DEC HILFSREG
		sz 
		  jmp Repeat_RD
		;---
EndBissData
		clrb SCK_SX			;ALE Clock zurcksetzen
		nop
		nop
		setb Enc0_CS			;Deaktivieren aller Biss Interface Bausteine
		nop
		setb Enc1_CS

		mov FSR,#CANS_MSG_DATA0
		mov INDF,#$E0
		setb SCK_SX			;ALE Clock setzen
ret

;---------------------------------------------------------------

;-Subroutine
EncBissSensorDat
		;Auslesen EOT 
		mov FSR,#MB3z_Befehl		;Lesen Register
		mov INDF,#$03

		mov FSR,#MB3z_Adr
		mov INDF,#$F0			;Statusmeldung (Reg 240)

		mov FSR,#StatusSensorDat
		mov INDF,#$78			;EOT ERROR, Break ERROR setzen, GetSens Error, nERR Error setzen
	
		;Anzahl zu bearbeitender Daten
		mov FSR,#MB3z_DatCount
		mov INDF,#$01			;1 Wort lesen/schreiben
		;---------------------------------

Check_EOT
		mov FSR,#StatusSensorDat	;Zhlerstand kontrollieren (Bit0 ..Bit2)
		snb INDF.2
                    jmp EndBissSensDat		;Sprung zum Ende der Routine

		    call EncBissData		;Lesen von EOT (Reg 240)

		mov FSR,#StatusSensorDat	;Zhler um 1 erhhen
		INC INDF

		mov FSR,#COUNTER0
		mov INDF,#$02
		page $E00
		     call TimeEater1		;Zeitverzgerung 
		page $800

		mov FSR,#MB3z_DatW0		;berprfen des Registerwertes
		snb INDF.0			
                   jmp Read_SensorD			;Wenn 1 Sprung zum nchsten Schritt
		;------

		mov FSR,#StatusSensorDat 	;berprfen ob Break bereits gesetzt wurde
		snb INDF.7
		    jmp Check_EOT		;Wenn BREAK bereits gesetzt wurde weiteres berprfen
		
		mov FSR,#MB3z_Befehl		;Schreiben Register
		mov INDF,#$02

		mov FSR,#MB3z_Adr
		mov INDF,#$F4			;Befehlsregister (Reg 244)

		mov FSR,#MB3z_DatW0
		mov INDF,#$80			;Auslsen eines Break

		    call EncBissData		;Schreiben eines Break ins Register

		mov FSR,#MB3z_Adr		;Adresse wieder auf Statusregister
		mov INDF,#$F0			;Statusmeldung (Reg 240)

		mov FSR,#MB3z_Befehl		;Lesen Register
		mov INDF,#$03
		    jmp Check_EOT
		;-------------------------------

Read_SensorD
		mov FSR,#StatusSensorDat	;Lschen des Break Errors und Counterwert
		mov INDF,#$78

		mov FSR,#MB3z_Befehl		;Schreiben des Register
		mov INDF,#$02
			
		mov FSR,#MB3z_Adr
		mov INDF,#$F4			;Befehlsregister (Reg 244)

		mov FSR,#MB3z_DatW0
		mov INDF,#$04			;Setzen von Get_Sens

		;Anzahl zu bearbeitender Daten
		mov FSR,#MB3z_DatCount
		mov INDF,#$01			;1 Wort schreiben

		    call EncBissData

		mov FSR,#MB3z_Befehl		;Lesen Register
		mov INDF,#$03

Wait_GetSens
		mov FSR,#StatusSensorDat	;Zhlerstand kontrollieren (Bit0 ..Bit2)
		snb INDF.2
                    jmp EndBissSensDat		;Sprung zum Ende der Routine

		mov FSR,#COUNTER0
		mov INDF,#$08
		page $E00
		     call TimeEater1		;Zeitverzgerung
		page $800
		    call EncBissData		;Auslesen des Befehlsregisters
				
		mov FSR,#StatusSensorDat
		INC INDF			;Zhler um 1 erhhen

		mov FSR,#MB3z_DatW0
		snb INDF.3
		    jmp Wait_GetSens		;Warten bis GetSens = 0
	
		;---
		mov FSR,#StatusSensorDat	;Lschen des GetSens Errors und Counterwert
		mov INDF,#$58
		
		mov FSR,#MB3z_Adr
		mov INDF,#$F0			;Statusmeldung (Reg 240)
		;---

Wait_EOT
		mov FSR,#StatusSensorDat	;Zhlerstand kontrollieren
		snb INDF.2
                    jmp EndBissSensDat		;Sprung zum Ende der Routine

		mov FSR,#COUNTER0
		mov INDF,#$02
		page $E00
		     call TimeEater1	
		page $800
		    call EncBissData		;Auslesen der
				
		mov FSR,#StatusSensorDat
		INC INDF			;Zhler um 1 erhhen

		mov FSR,#MB3z_DatW0
		sb INDF.0
		   jmp Wait_EOT			;Warten bis EOT = 1

		mov FSR,#StatusSensorDat	;Lschen des EOT Errors und Counterwert
		mov INDF,#$48

Check_nERR
		mov FSR,#MB3z_DatW0 
		sb INDF.7
                   jmp EndBissSensDat		;FehlerFall nErr  = 0 
  
		mov FSR,#StatusSensorDat	;Lschen des CheckErr Errors und Counterwert
		mov INDF,#$40	

SensorData
		mov FSR,#MB3z_Adr
		mov INDF,#$00			;Statusmeldung (Reg 240)
		mov FSR,#MB3z_DatCount
		mov INDF,#$02

		    call EncBissData

		mov FSR,#MB3z_DatW0
		mov HILFSREG,INDF
		and HILFSREG,#$03
		cjne HILFSREG,#$03,EndRoutSSD	;Ungltige Daten
 
		;=================================================================
		;berprfung Drehrichtung
CheckDirection
		mov FSR,#Direction
		sb INDF.7		
                   jmp Neg_Lauf				;Negativer Motorlauf
		jmp Pos_Lauf				;Positiver Motorlauf
		;---------------------------

Neg_Lauf	;Linkskslauf (-)
		mov FSR,#SensPosD1
		and INDF,#$1F				;Maskieren auf relevanten Teil
		mov StartVar,INDF

		mov FSR,#MB3z_DatW1
		and INDF,#$1F
		mov EndVar,INDF

		cjbe EndVar,StartVar,Copy_IndexPos	;Wenn letzte Wert < voriger Wert ==> Verlassen der Routine (vorher kopieren des Sensorwertes)
		sub EndVar,StartVar
		cjb EndVar,#$08,Copy_IndexPos		;Wenn Delta < 08 (512)==> Verlassen der Routine (vorher kopieren des Sensorwertes)			
		   call	L_subpos			;sonst Index -1

		mov FSR,#StatusSensorDat		;Setzen des Counterwertes (Index Vernderung stattgefunden)
		mov INDF,#$41	
                   jmp Copy_IndexPos
		;---

Pos_Lauf	;Rechtslauf (+)
		;berprfen ob Index verndert?
		mov FSR,#SensPosD1
		and INDF,#$1F				;Maskieren auf relevanten Teil
		mov StartVar,INDF

		mov FSR,#MB3z_DatW1
		and INDF,#$1F
		mov EndVar,INDF
	
		cjae EndVar,StartVar,Copy_IndexPos	;Wenn letzte Wert > voriger Wert ==> Verlassen der Routine (vorher kopieren des Sensorwertes)			
		sub StartVar,EndVar
		cjb StartVar,#$08,Copy_IndexPos		;Wenn Delta < 08 (512) ==> Verlassen der Routine (vorher kopieren des Sensorwertes)			
		   call L_addpos			;dann 

		mov FSR,#StatusSensorDat		;Setzen des Counterwertes (Index Vernderung stattgefunden)
		mov INDF,#$41	
		;---------------------------
Copy_IndexPos
		;bernehmen der neuen Daten in Speicher
		mov StartVar,#MB3z_DatW0		;IndexPosD0 2..		
		mov EndVar,#SensPosD0		;Informationen_Data 0..
		mov FSR,#CountCopy
		mov INDF,#$02

		page $E00
		     call DataCopy
		page $800

		mov FSR,#StatusSensorDat	;Lschen des CheckErr Errors und Counterwert
		cjne INDF,#$41,EndRoutSSD	;Wenn keine Index-Erhhung stattgefunden hat ==> Beenden der Routine

		;Kopieren IndexPos und SensPos in CAN MSG
		mov StartVar,#IndexPosD0		;IndexPosD0 2..		
		mov EndVar,#CANS_MSG_DATA1		;Informationen_Data 0..
		mov FSR,#CountCopy
		mov INDF,#$05

		page $E00
		     call DataCopy
		page $800

		mov FSR,#CANS_MSG_DBYTE
		mov INDF,#$07

EndBissSensDat
		mov FSR,#CANS_MSG_DATA0
		mov INDF,#$E5

		mov FSR,#StatusSensorDat	;berprfung Sensor Daten
		mov HILFSREG,INDF

		mov FSR,#CANS_MSG_DATA6
		mov INDF,HILFSREG
		sb HILFSREG.3
		   jmp SenSensData

		mov FSR,#CANS_MSG_DBYTE
		mov INDF,#$02

		mov FSR,#StatusSensorDat	;Sensor Daten
		mov HILFSREG,INDF
		clr INDF
		mov FSR,#CANS_MSG_DATA1		;Informationen_Data 1
		mov INDF,HILFSREG
		 
SenSensData
		page $C00
		     call SendCanMsg 
		page $800

EndRoutSSD
		mov FSR,#StatusSensorDat
		and INDF,#$40
ret
;----------------------------------------------------


;-Routinen fr Addition bzw. Subtraktion der Position
addpos		;Addition des Indexcounters (24 bit)
		;Addition mit 1
		mov FSR,#IndexPosD0	;Positionsbyte 0
		add INDF,#$01
        	sc
	   	  jmp end_addpos
ADD_HByte
		inc FSR 		;Positionsbyte 1
		add INDF,#$01
		sc
		   jmp end_addpos

		inc FSR 		;Positionsbyte 2
		add INDF,#$01
end_addpos

ret
;----------------------------------------------------

subpos		;Subtraktion des Indexcounters (24 bit)
		;Subtraktion mit 1
		mov FSR,#IndexPosD0	;Positionsbyte 0
		sub INDF,#$01
      	  	snc
		   jmp end_subpos
SUB_HByte
		inc FSR 		;Positionsbyte 1
		sub INDF,#$01

		snc			;wenn Ergebnis negativ, wird C Flag zurckgesetzt
		   jmp end_subpos

		inc FSR 		;Positionsbyte 2
		sub INDF,#$01
end_subpos
ret
;----------------------------------------------------

CheckSensMsg
		mov FSR,#CANE_MSG_DATA6
		mov HILFSREG,INDF

		mov FSR,#CANS_MSG_DATA1		;Informationen_Data 1
		mov INDF,HILFSREG

		mov FSR,#StatusSensorDat
		mov INDF,HILFSREG
		and INDF,#$40			;Maskieren auf Sensordatenanfrage
	
		mov FSR,#MB3z_ICSelc
		mov INDF,HILFSREG
		and INDF,#$01			;Maskieren auf IC_Select

		mov FSR,#Direction
		mov INDF,HILFSREG
		and INDF,#$80

		cje HILFSREG,#$00,CopyLastData	;wenn Gleich Kopieren der aktuellen Werte

		;Auslesen der Eingangsdaten
		mov StartVar,#CANE_MSG_DATA1	;IndexPosD0 2..		
		mov EndVar,#IndexPosD0		;Informationen_Data 0..
		mov FSR,#CountCopy
		mov INDF,#$05

		page $E00
		     call DataCopy
		page $800
     ret

CopyLastData
		;Schreiben der letzten Daten
		mov StartVar,#IndexPosD0	;IndexPosD0 2..		
		mov EndVar,#CANS_MSG_DATA1	;Informationen_Data 0..
		mov FSR,#CountCopy
		mov INDF,#$05

		page $E00
		     call DataCopy
		page $800

SendSensEndMsg
		page $C00
		     call SendCanMsg 
		page $800
ret
;----------------------------------------------------


