;==========================================================================================
; Programm mit Subroutinen fr die Encoder
;
; Visitec GmbH Februar 2008 by N.Fritsch   	     
; 
; Encoder.inc
;==========================================================================================
;
; DEFINIERTE ROUTINEN SIND:
;
;	EncBissData:	Subroutine zum Lesen/Schreiben von Daten in bzw. aus dem IC_MB3 zur Kommunikation mit Encodern
;			es sind 1..5 Datenbytes bertragbar
;
;	READSENS:	Subroutine zum Auslesen der Sensordaten aus IC_MB3 (2 Datenbytes -> SensPosD1, SensPosD0)
;			Dabei wird Bit UCREADSENS zunchst gesetzt, anschlieend werden die Sensordaten ausgelesen und 
;			abschlieend wird das BitUCreadSens wieder zurckgesetz. Sollte ein Fehler auftreten (Bit(1:0) so wird 
;			der Fehler in CANS_MSG_DATA1 abgespeichert und das Register Fehler_Count enthlt den Wert $00. 
;			Tritt kein Fehler auf so hat dieses Byte den Wet $03. 
;
;	WaitEOT:	Subroutine zum Auslesen des Befehlsregisters und zur berprfung, das EOT = 1
;
;	AGSStart:	Subroutine zur Aktivierung des AGS Mode und Auslesen der ersten Sensordaten
;
;	AGSMode:	Subroutine zum Regelmigen Auslesen der Sensordaten und Erhhung bzw. Verringerung der Indexposition
;
;	
;
; Verwendung im Hauptprogramm: 	include "Encoder_V5.inc"
;
; Bemerkungen:	----		
;
; Speicherbereich($800-$9CB)
; Lnge ($1CC = 460) Speicherwrter
;-------------------------------------------------------------------------------------------
;		s e r i e l l e   K o m u n i k a t i o n   m i t   E n c o d e r
;===========================================================================================
;Fr Biss Interface:
;-------------------
Enc0_CS		equ RA.3	;SX.OUT_A3 	(Enc1_CS)
Enc0_EOT	equ RA.4	;SX.IN_A4  	(Enc1_EOT)
Enc0_NER	equ RA.5	;SX.IN/OUT_A5	(Enc1_NER)

Enc1_CS		equ RB.1	;SX.OUT_B1 	(Enc2_CS)
Enc1_EOT	equ RB.2	;SX.IN_B2  	(Enc2_EOT)
Enc1_NER	equ RB.3	;SX.IN/OUT_B3	(Enc2_NER)

Enc_GetSens	equ RB.6	;SX.OUT_B6 	(Enc_GetSens)
Enc_NRes	equ RB.7	;SX.OUT_B7  	(Enc_NRes)


;Konstantendefinition fuer Biss Interface
Ins_BissWriteData	equ	%00000010
Ins_BissReadData	equ	%00000011
Ins_BissReadStatus	equ	%00000101
Ins_BissWriteInstr	equ	%00000111

;Definition der Register des IC MB3
MB3_BefehlsReg		equ	$F4
MB3_StatusReg		equ	$F0



;Variablendefinition fr Biss Interface
MB3z_ICSelc	equ	$50
MB3z_Befehl	equ	$51
MB3z_Adr	equ	$52
MB3z_DatW0	equ	$53
MB3z_DatW1	equ	$54
MB3z_DatW2	equ	$55
MB3z_DatW3	equ	$56
MB3z_DatW4	equ	$57
MB3z_DatW5	equ	$58
MB3z_DatCount	equ	$59

SensPosD0	equ	$5A
SensPosD1	equ	$5B

IndexPosD0	equ	$5C
IndexPosD1	equ	$5D
IndexPosD2	equ	$5E

IndexPosI1D0	equ	$60
IndexPosI1D1	equ	$61
IndexPosI1D2	equ	$62

IndexPosI2D0	equ	$63
IndexPosI2D1	equ	$64
IndexPosI2D2	equ	$65

Fehler_Count	equ	$66
StatusSensorDat equ	$67

Bereichs_Mark	equ	$68
Bereichs_MarkI1 equ	$69
Bereichs_MarkI2 equ	$6A

AGSReadCounter	equ	$6B
AGSReadEndVal	equ	$6C
;oberer Bereich $10
;unterer Bereich $01
;-------------------------------------------------------------------------------------------


;===========================================================================================
;-Speicheradresse fr Programmcode-
org $800	

;Definition von Sprungadressen fr Subroutinen
L_Subpos		jmp SubPos
L_Addpos		jmp AddPos
L_AGSStart		jmp AGSStart
L_AGSMode		jmp AGSMode
L_UnSelectMB3		jmp UnSelectMB3
L_SelectMB3		jmp SelectMB3

;-------------------------------------------------------------------------------------------
;-Subroutine
EncBissData
		clrb SCK_SX			;ALE Clock zurcksetzen
		mov FSR,#MB3z_ICSelc		;Auswahl des entsprechenden BISS Interface Bausteins
		csne INDF,#$00			;berprfen ob BISS Interface 0 beschrieben werden soll
		    clrb Enc0_CS			;aktivieren BISS Interface 0
		
		csne INDF,#$01			;berprfen ob BISS Interface 1 beschrieben werden soll
		    clrb Enc1_CS			;aktivieren BISS Interface 1
		
		csne INDF,#$02			;berprfen ob beide BISS Interface beschrieben werden sollen	
    		    clrb Enc0_CS			;aktivieren BISS Interface 0
		csne INDF,#$02
    		    clrb Enc1_CS			;aktivieren BISS Interface 1

		mov FSR,#MB3z_Befehl		;Senden des Befehls
		page $C00
		     call sendbyte
		page $800

		;Laden der Anzahl zu bearbeitender Daten
		mov FSR,#MB3z_DatCount
		mov HILFSREG,INDF
		
		;Ueberpruefung ob Befehl Read Status
		mov FSR,#MB3z_Befehl
		cje INDF,#Ins_BissReadStatus,Read_BissDat	
		cje INDF,#Ins_BissWriteInstr,Write_BissDat

		mov FSR,#MB3z_Adr
		page $C00
		     call sendbyte
		page $800

		;Ueberpruefung ob Read Befehl
		mov FSR,#MB3z_Befehl
		cje INDF,#Ins_BissReadData,Read_BissDat	
		;---
Write_BissDat
		;Schreiben von Daten
		mov FSR,#MB3z_DatW0
Repeat_WR		
		page $C00
		     call sendbyte
		page $800

		INC FSR				;Zeiger auf eine Adresse weiter stellen 	
		DEC HILFSREG
		sz 
		  jmp Repeat_WR	

		jmp EndBissData
		;---
Read_BissDat
		mov FSR,#MB3z_DatW0
Repeat_RD		
		page $C00
		     call getbyte
		page $800

		INC FSR				;Zeiger auf eine Adresse weiter stellen 	
		DEC HILFSREG
		sz 
		  jmp Repeat_RD
		;---
EndBissData
		clrb SCK_SX			;ALE Clock zurcksetzen
		nop
		nop
		setb Enc0_CS			;Deaktivieren aller Biss Interface Bausteine
		nop
		nop
		setb Enc1_CS
		nop
		nop
		setb SCK_SX			;ALE Clock setzen
ret

;---------------------------------------------------------------

;-Subroutine
;---Auslesen der Sensordaten aus IC_MB3---
READSENS
		mov FSR,#Fehler_Count		;Fehler-Counter auf OKAY setzen
		mov INDF,#$03

		;-Setzen von UCREADSENS-
		mov FSR,#MB3z_DatCount		;Setzen der Anzahl zu lesender Daten
		mov INDF,#$01
		mov FSR,#MB3z_Befehl		;Laden des Read Befehls
		mov INDF,#Ins_BissReadData
		INC FSR				;Laden der Adresse
		mov INDF,#MB3_BefehlsReg
	    	    call EncBissData		;Auslesen des Inhalts des Befehlsregisters
		;-----
		mov FSR,#MB3z_Befehl		;Laden des Write Befehls
		mov INDF,#Ins_BissWriteData
		INC FSR				;Laden der Adresse
		mov INDF,#MB3_BefehlsReg
		INC FSR				;MB3z_DatW0
		or INDF,#$40			;Setzen von UCREADSENS
		    call EncBissData		;Setzen des Inhalts des Befehlsregisters
		;-----------------------

		;Auslesen der Sensordaten
		mov FSR,#MB3z_Befehl		;Laden des Read Befehls
		mov INDF,#Ins_BissReadData
		INC FSR				;Laden der Adresse (MB3z_Adr)
		mov INDF,#$00			;setzen auf Sensordaten Register
		mov FSR,#MB3z_DatCount
		mov INDF,#$02			;MB3z_DatW0, MB3z_DatW1
		    call EncBissData
		;-----------------------

		;bernehmen der neuen Daten in Speicher
		mov StartVar,#MB3z_DatW0	;MB3z_DatW0..1		
		mov EndVar,#SensPosD0		;SensPosD0..1
		mov FSR,#CountCopy
		mov INDF,#$02
		page $E00
		     call DataCopy
		page $800
		;-----------------------

		;-Zurcksetzen von UCREADSENS-
		mov FSR,#MB3z_DatCount
		mov INDF,#$01
		mov FSR,#MB3z_Befehl		;Laden des Read Befehls
		mov INDF,#Ins_BissReadData
		INC FSR				;Laden der Adresse
		mov INDF,#MB3_BefehlsReg
	    	    call EncBissData		;Auslesen des Inhalts des Befehlsregisters
		;----
		mov FSR,#MB3z_Befehl		;Laden des Write Befehls
		mov INDF,#Ins_BissWriteData
		INC FSR				;Laden der Adresse
		mov INDF,#MB3_BefehlsReg
		INC FSR				;MB3z_DatW0
		and INDF,#$BF			;Zurcksetzen von UCREADSENS
		    call EncBissData		;Setzen des Inhalts des Befehlsregisters
		;-----------------------

		;-Auswerten der Fehlerbits
		mov FSR,#SensPosD0		;Informationen_Data 0
		mov HILFSREG,INDF
		and HILFSREG,#$03
		cje HILFSREG,#$03,EndRoutSSD	;Wenn Fehlerbits ungleich 3, dann Fehler in Daten (Ungltige Daten)

		;bernehmen der FehlerBits
		mov FSR,#CANS_MSG_DATA2		;bernehmen der Fehlerbits der Nachricht
		mov INDF,HILFSREG 
		mov FSR,#Fehler_Count		;Fehler bei 00
		clr INDF
		;-----------------------
EndRoutSSD
		nop	
ret
;----------------------------------------------------

;-Subroutine
;---berprfen des EOT Wertes---
WaitEOT
		mov FSR,#Fehler_Count		;Fehler-Counter auf OKAY setzen
		mov INDF,#$03
Repeat_EOT
		;-Zeitverzgerung-
		mov FSR,#COUNTER0
		mov INDF,#$08
		page $E00
		     call TimeEater1	
		page $800
		;-----------------------

		;-Auslesen EOT-
		mov FSR,#MB3z_DatCount
		mov INDF,#$01
		mov FSR,#MB3z_Befehl		;Laden des Read Befehls
		mov INDF,#Ins_BissReadData
		INC FSR				;Laden der Adresse
		mov INDF,#MB3_StatusReg		
	    	    call EncBissData		;Auslesen des Inhalts des Status Registers
		;----
		mov FSR,#MB3z_DatW0
		and INDF,#$01			;Maskieren von EOT
		;-----------------------

		cje INDF,#$01,EOT_READY		;Wenn EOT=1 beenden der Routine

		mov FSR,#Fehler_Count		;Fehler_Counter Dekrementieren
		DEC INDF
		sz
		    jmp Repeat_EOT		;Sprung zum Anfang der Routine
		;-----------------------
		mov FSR,#CANS_MSG_DATA2
		mov INDF,#$10			;Fehler EOT
EOT_READY
ret
;----------------------------------------------------

;-Subroutine
;---bernehmen der Daten des ausgewhlten IC MB_3---
SelectMB3	
		mov FSR,#MB3z_ICSelc		;IC auswhlen
		    cje INDF,#$01,LoadIC2	;(0=IC1, 1=IC2)

LoadIC1
		mov FSR,#CANS_MSG_DATA1
		mov INDF,#$40			;Statusbyte IC1
		
		;-Daten bernehmen-
		mov StartVar,#IndexPosI1D0	;IndexPosI1D0..2		
		mov EndVar,#IndexPosD0		;IndexPosD0..2
		mov FSR,#CountCopy
		mov INDF,#$03
		page $E00
		     call DataCopy
		page $800

		;Kopieren MarkI1 => Mark
		mov FSR,#Bereichs_MarkI1
		mov HILFSREG,INDF
		mov FSR,#Bereichs_Mark
		mov INDF,HILFSREG
		    ret				;Routine beenden
		;-----------------------

LoadIC2
		mov FSR,#CANS_MSG_DATA1
		mov INDF,#$41			;Statusbyte IC1

		;Daten bernehmen
		mov StartVar,#IndexPosI2D0	;IndexPosI2D0..2		
		mov EndVar,#IndexPosD0		;IndexPosD0..2
		mov FSR,#CountCopy
		mov INDF,#$03
		page $E00
		     call DataCopy
		page $800

		;Kopieren MarkI2 => Mark
		mov FSR,#Bereichs_MarkI2
		mov HILFSREG,INDF
		mov FSR,#Bereichs_Mark
		mov INDF,HILFSREG
ret
;----------------------------------------------------

;-Subroutine
;---Sichern der Daten des ausgewhlten IC MB_3---	
UnSelectMB3
		mov FSR,#MB3z_ICSelc		;IC auswhlen
		    cje INDF,#$01,SaveIC2	;(0=IC1, 1=IC2)

SaveIC1		
		;Daten bernehmen
		mov StartVar,#IndexPosD0	;IndexPosD0..2		
		mov EndVar,#IndexPosI1D0	;IndexPosI1D0..2
		mov FSR,#CountCopy
		mov INDF,#$03
		page $E00
		     call DataCopy
		page $800

		;Kopieren Mark => MarkI1
		mov FSR,#Bereichs_Mark
		mov HILFSREG,INDF
		mov FSR,#Bereichs_MarkI1
		mov INDF,HILFSREG
		    ret				;Routine beenden
		;-----------------------

SaveIC2
		;Daten bernehmen
		mov StartVar,#IndexPosD0	;IndexPosD0..2		
		mov EndVar,#IndexPosI2D0	;IndexPosI1D0..2
		mov FSR,#CountCopy
		mov INDF,#$03
		page $E00
		     call DataCopy
		page $800

		;Kopieren Mark => MarkI2
		mov FSR,#Bereichs_Mark		
		mov HILFSREG,INDF
		mov FSR,#Bereichs_MarkI2
		mov INDF,HILFSREG
ret
;----------------------------------------------------

;-Subroutine
;---Aktivierung von AGS---
AGSStart
		mov FSR,#MB3z_ICSelc		;berprfen MB3z_ICSelc
		    cje INDF,#$01,AktivIC2	;Wenn 1 => Auswahl IC2
		;---		
AktivIC1
		mov FSR,#CANS_MSG_DATA1
		mov INDF,#$40			;Statusbyte IC1
		    jmp AktivAGS
		;---
AktivIC2
		mov FSR,#CANS_MSG_DATA1
		mov INDF,#$41			;Statusbyte IC1
		;---
AktivAGS
		;Auslesen der Eingangsdaten
		mov StartVar,#CANE_MSG_DATA2	;CANE_MSG_DATA2..4		
		mov EndVar,#IndexPosD0		;IndexPosD0..2
		mov FSR,#CountCopy
		mov INDF,#$03
		page $E00
		     call DataCopy
		page $800
		;-----------------------

		;-Aktiviern AGS-
		;Setzen der Anzahl zu bearbeitender Daten
		mov FSR,#MB3z_DatCount		;Anzahl der auszulesenden Daten
		mov INDF,#$01
		mov FSR,#MB3z_Befehl		;Laden des Write Befehls
		mov INDF,#Ins_BissWriteData
		INC FSR				;Laden der Adresse
		mov INDF,#MB3_BefehlsReg	;Adresse F4
		INC FSR				;Setzen des Inhaltes auf (AGS)
		mov INDF,#$01

		    call EncBissData		;Aufrufen der Routine zum Zugriff auf IC_MB3
		    call WaitEOT		;Aufruf warten auf EOT Ready

		;berprfung EOT Fehler
		mov FSR,#Fehler_Count
		cje INDF,#$00,SEND_AGSDATA	;Fehler EOT senden
		;---
		    call READSENS		;Auslesen der Sensordaten
	
		;berprfung Fehler SENS Data
		mov FSR,#Fehler_Count
		cje INDF,#$00,SEND_AGSDATA      ;Fehler Sensordaten senden
		;-----------------------
		
		;bernehmen der neuen Daten in Speicher 
		mov StartVar,#SensPosD0		;Informationen_Data 0..
		mov EndVar,#CANS_MSG_DATA2
		mov FSR,#CountCopy
		mov INDF,#$02
		page $E00
		     call DataCopy
		page $800
		mov FSR,#CANS_MSG_DBYTE			;Anzahl Datenbytes auf 4 setzen
		mov INDF,#$04
		    jmp SEND_AGSDATA
		;-----------------------
		
SEND_AGSDATA	;Senden der Can Nachricht
		page $C00
		     call SendCanMsg 
		page $800
		mov FSR,#Bereichs_Mark
		clr INDF

		;Daten in zugehrigen Speicher des jeweiligen IC's legen 
		    call L_UnSelectMB3
ret
;----------------------------------------------------



;=================================================================
AGSMode
		;Aufruf der Subroutine zum Laden der Daten des jeweiligen IC
		    call L_SelectMB3
		;---

		;Aufruf der Subroutine zum Lesen der Sensordaten
		    call READSENS
		;---
		;jmp Copy_IndexPos

		;berprfen ob Fehler vorhanden
		mov FSR,#Fehler_Count
		cje INDF,#$00,EndAGSMode		;Wenn Fehler Counter 0, dann Routine Beeenden		
		;---

		;berprfen der Positionsdaten
		mov FSR,#SensPosD1
		mov HILFSREG,INDF
		and HILFSREG,#$18				;Maskieren auf relevanten Teil Bits 8/7

		cje HILFSREG,#$18,CheckHighMark		;High setzen und berprfen ob Index -1
		cje HILFSREG,#$00,CheckLowMark		;Low setzen und berprfen ob Index +1
		mov FSR,#Bereichs_Mark			;Bereichsmark zurcksetzten
		clr INDF
		    jmp EndAGSMode			;Sonst Routine Beenden
		;-----------------------
  
		;---berprfung der wichtigen Bereiche der Sensordaten
CheckLowMark
		mov FSR,#Bereichs_Mark
		cjne INDF,#10,SetLowMark		;Prfen ob Bereichsmarker vorher im High Bereich war, wenn nicht Sprung zu Setzen

		;Addition der IndexPosition
		     call L_AddPos			;sonst Index +1
		mov FSR,#StatusSensorDat
		setb INDF.7				;Setzen des Bits, das Nachricht zu senden ist
SetLowMark
		mov FSR,#Bereichs_Mark			;Bereichsmarker auf Low Bereich setzen
		mov INDF,#$01
    		    jmp Copy_IndexPos			;Sprung zu Copy_IndexPos
		;-----------------------

CheckHighMark
		mov FSR,#Bereichs_Mark
		cjne INDF,#01,SetHighMark		;Prfen ob Bereichsmarker vorher im Low Bereich war, wenn nicht Sprung zu Setzen

		;Subtraktion der IndexPosition
		     call L_SubPos			;sonst Index -1
		mov FSR,#StatusSensorDat
		setb INDF.7				;Setzen des Bits, das Nachricht zu senden ist
SetHighMark
		mov FSR,#Bereichs_Mark			;Bereichsmarker auf High Bereich setzen
		mov INDF,#$10
		;-----------------------

Copy_IndexPos
		;Kopieren IndexPos und SensPos in CAN MSG
		mov StartVar,#SensPosD0			;SensPosD0..1 und IndexPosD0..2		
		mov EndVar,#CANS_MSG_DATA2		;Informationen_Data 0..
		mov FSR,#CountCopy
		mov INDF,#$05
		page $E00
		     call DataCopy
		page $800

		mov FSR,#CANS_MSG_DBYTE
		mov INDF,#$07
		INC FSR					;CANS_MSG_DATA0 mit $E6 laden
		mov INDF,#$E6
		;-----------------------
	
SenSensData
		mov FSR,#StatusSensorDat
		sb INDF.7			;Senden der Nachricht, wenn Bit7 = 1
		   jmp EndAGSMode 
		page $C00
		     call SendCanMsg 
		page $800
EndAGSMode
		mov FSR,#StatusSensorDat
		clrb INDF.7			;Bit das Nachricht zu senden ist lschen
		     call L_UnSelectMB3
ret
;----------------------------------------------------
;========================================================================================

;-Routinen fr Addition bzw. Subtraktion der Position
AddPos		;Addition des Indexcounters (24 bit)
		;Addition mit 1
		mov FSR,#IndexPosD0	;Positionsbyte 0
		add INDF,#$01
	       	sc
	   	  jmp end_addpos
ADD_HByte
		inc FSR 		;Positionsbyte 1
		add INDF,#$01
		sc
		   jmp end_addpos
		inc FSR 		;Positionsbyte 2
		add INDF,#$01
end_addpos
ret
;----------------------------------------------------

SubPos		;Subtraktion des Indexcounters (24 bit)
		;Subtraktion um 1
		mov FSR,#IndexPosD0	;Positionsbyte 0
		sub INDF,#$01
     	  	snc
		   jmp end_subpos
SUB_HByte
		inc FSR 		;Positionsbyte 1
		sub INDF,#$01
		snc			;wenn Ergebnis negativ, wird C Flag zurckgesetzt
		   jmp end_subpos
		inc FSR 		;Positionsbyte 2
		sub INDF,#$01
end_subpos
ret
;----------------------------------------------------
;====================================================


