;==========================================================================================
; Programm mit Subroutinen fr die Auswertung und das Senden von neuen Nachrichten
;
; Visitec GmbH Februar 2007 by N.Fritsch   	     
; 
; ParseMsgMot_V2.inc
;==========================================================================================
;
; DEFINIERTE ROUTINEN SIND:
;
;	parseCanMsg:	Routine zur Auswertung der Can Nachrichten fr die Motorsteuerung
;			es werden in dieser Routine alle notwendigen Aufgaben gesteuert, diese sind:
;
;				->	Tri_ReadOut	Auslesen eines Trinamics
;			
;				->	Tri_WriteTo	Beschreiben eines Trinamics
;
;				->	Tri_FiFo	Schreiben in Positions FiFo eines Trinamics
;
;				->	RD_EEPROM	Lesen des EEPROM
;
;				->	WR_EEPROM	Schreiben in EEPROM
;
;				->	RD_STEEPROM	Lesen des Status-Registers des EEPROM
;
;				->	WR_STEEPROM	Schreiben in Status-Register des EEPROM
;
;				->	WR_EEPROM_DIS	Deaktivieren des Schreibzugriffs auf EEPROM
;
;				->	WR_EEPROM_ENA	Aktivieren des Schreibzugriffs auf EEPROM
;
;				->	SaveTriState	Speichern der Motorpositionen aus Trinamic in EEPROM
;				
;				->	ReadTriState	Lesen der Motorpositionen aus EEPROM in Trinamic
;
;				->	RTriRegister	Lesen der aktuellen Motor-Geschwindigkeit od. Motor-Position od. Encoderwerte aus Trinamic
;
;				->	RequestReady	Senden der Begrungsnachricht
;
;				->	RequestReadPort Senden des Status aller Ports
;
;				->	RequestMotDrive	Sicherheitsnachricht beim Fahren der Motoren
;
;				->	PowEndSwitch	Einschalten der Power fr Endschalter
;
;				->	WEndStufen	Setzen der Microsteps fr Endstufen
;
;				->	CanBaseAdr1	Unter dieser Routine wird je nach Wert des Datenbytes 0 die TypID, der Status der SX Ports
;							oder die Ready Nachricht gesendet
;
;	ReadEepromToTri:	Auslesen der aktuellen Motoren-Position aus Trinamics und Sichern in EEPROM 

;	SaveTriToEeprom:	Auslesen der abgesicherten Positionen aus EEPROM und schreiben in Trinamics

;	SAVE_POS:		Unterfunktion zu SaveTriToEeprom

;	READ_POS:		Unterfunktion zu ReadEepromToTri
;			
;
; Verwendung im Hauptprogramm: 	include "ParseMsgMot.inc"
;
; Bemerkungen:			Vor Laden dieses include-Files sind die Trinamics Routinen, die EEPROM Routinen 
;				und die MCP2515 Routinen zu laden.
;
; Speicherbereich($200-$3FC)
; Speicherbereich($400-$502)
; Lnge ($2FE = 766) Speicherwrter
;---------------------------------------------------------------------------------------------
;		A U S W E R T U N G  V O N  N A C H R I C H T E N
;=============================================================================================

;-Definition von SX Steuerleitungen-
LED_CS		equ	RC.3
Power_EndS	equ	RA.0
;-Konstante-

;-------------------------------------------------------------------------------------------------------------
;=============================================================================================================

;-Macros-

;===================================================================================
;-Speicheradresse fr Programmcode-
org $200  


;-Subroutine-
;------------ Routine zur Auswertung der CAN MSG ---------------
parseCanMsg		
		page $400
		mov FSR,#CANE_MSG_ADDRH
		cjne INDF,#BaseAdrCanRXH,parseCanEnd	;Pruefen des High Bzte
		
		mov FSR,#CANE_MSG_ADDRL
		cjne INDF,#BaseAdrCanRXL,parseCanEnd	;Pruefen des LOW Byte

		mov FSR,#CANE_MSG_DBYTE
		cjb INDF,#$01,parseCanEnd 		;Auf Lnge prfen, wenn kleiner 1 dann beenden

CanBaseAdr	
		page $200						
		mov FSR,#CANE_MSG_DATA0			;Command identifizieren
		
		cje INDF,#$F0,RD_EEPROM
		cje INDF,#$F1,WR_EEPROM
		cje INDF,#$F2,RD_STEEPROM
		cje INDF,#$F3,WR_STEEPROM
		cje INDF,#$F4,WR_EEPROM_DIS
		cje INDF,#$F5,WR_EEPROM_ENA
		cje INDF,#$F6,SaveTriState
		cje INDF,#$F7,ReadTriState

		cje INDF,#$E2,RequestMotDrive
		cje INDF,#$F9,PowEndSwitch
		cje INDF,#$FA,WEndStufen
		cje INDF,#$FB,RTriRegister		
		cje INDF,#$FC,Tri_ReadOut
		cje INDF,#$FD,Tri_FiFo
		
		page $400
		    cje INDF,#$FE,Tri_WriteTo
		    cje INDF,#$FF,RequestReady
		    cje INDF,#$00,RequestReadPort
		    cje INDF,#$E0,BissAbsEncoder
		    cje INDF,#$E1,BissPorts
   		    cje INDF,#$E5,Sensor_Data
		    cje INDF,#$E3,SetEndVorCM    		
		    jmp parseCanEnd
		;=====================================
		;/////////////////////////////////////


RD_EEPROM	;**** Aus EEPROM lesen (#$F0) ****
		mov FSR,#CANS_MSG_DATA0			;ReadOut-Code fr abgehende MSG setzen		(DATA 0)
		mov INDF,#$F0					

		mov FSR,#CANE_MSG_DATA1            	;Adress-Register lesen
		mov HILFSREG,INDF
		mov FSR,#CANS_MSG_DATA1			;Adress-Register fr abgehende MSG setzen	(DATA 1)
		mov INDF,HILFSREG
		mov FSR,#EEPROM_ADR			;Adresse fr EEPROM setzen
		mov INDF,HILFSREG

		mov FSR,#EEPROM_COUNT			;Anzahl der zu lesenden Bytes angeben
		mov INDF,#$01
			
		mov FSR,#EEPROM_INSTR			;Wert des Instruction- Registers setzen
		mov INDF,#EEPROM_RDFL

		page $E00
		     call RD_EEPROMSP
		page $200
						
		mov FSR,#EEPROM_DATA0
		mov HILFSREG,INDF	
		mov FSR,#CANS_MSG_DATA2			;Adress-Register fr abgehende MSG setzen	(DATA 2)
		mov INDF,HILFSREG

		mov FSR,#CANS_MSG_DBYTE			;Lnge fr MSG setzen 				(DBYTE)
		mov INDF,#$03
				
		page $C00
		     call SendCanMsg 
		page $400
		     jmp parseCanEnd
		;=====================================


WR_EEPROM	;**** In EEPROM schreiben (#$F1) ****
		mov StartVar,#CANE_MSG_DATA0		;Empfangs_Data 0..2		
		mov EndVar,#CANS_MSG_DATA0		;Sende_Data 0..2
		mov FSR,#CountCopy
		mov INDF,#$03

		page $E00
		     call DataCopy
		page $200

		mov FSR,#CANE_MSG_DATA1			;Empfangs_Data 1		
		mov HILFSREG,INDF
		mov FSR,#EEPROM_ADR			;Adresse fr EEPROM
		mov INDF,HILFSREG

		mov FSR,#CANE_MSG_DATA1			;Empfangs_Data 1		
		mov HILFSREG,INDF
		mov FSR,#EEPROM_DATA0			;Daten fr EEPROM
		mov INDF,HILFSREG

		mov FSR,#EEPROM_COUNT			;Anzahl der zu lesenden Bytes angeben
		mov INDF,#$01
			
		mov FSR,#EEPROM_INSTR			;Wert des Instruction- Registers setzen
		mov INDF,#EEPROM_WRFL		

		page $E00
		     call WR_EEPROMSP
		page $200
						
		mov FSR,#CANS_MSG_DBYTE			;Lnge fr MSG setzen 				(DBYTE)
		mov INDF,#$03
				
		page $C00
		     call SendCanMsg 
		page $400
		     jmp parseCanEnd
		;=====================================


RD_STEEPROM	;**** StatusRegister aus EEPROM lesen (#$F2) ****
		mov FSR,#CANS_MSG_DATA0			;ReadOut-Code fr abgehende MSG setzen		(DATA 0)
		mov INDF,#$F2	
							
		page $E00
		     call RD_EEPROMSR			
		page $200

		mov FSR,#EEPROM_DATA0			;EEPROM- Datenbyte des Status-Registers auslesen
		mov HILFSREG,INDF
		mov FSR,#CANS_MSG_DATA1			;Datenbyte in abgehende MSG setzen		(DATA 1)
		mov INDF,HILFSREG

		mov FSR,#CANS_MSG_DBYTE			;Lnge fr MSG setzen 				(DBYTE)
		mov INDF,#$02
								
		page $C00
		     call SendCanMsg 
		page $400
		     jmp parseCanEnd
		;=====================================


WR_STEEPROM	;**** StatusRegister in EEPROM schreiben (#$F3) ****
		mov FSR,#CANS_MSG_DATA0			;ReadOut-Code fr abgehende MSG setzen		(DATA 0)
		mov INDF,#$F3	
			
		mov FSR,#CANE_MSG_DATA1			;Datenbyte in abgehende MSG setzen		(DATA 1)
		mov HILFSREG,INDF
		mov FSR,#EEPROM_DATA0			;EEPROM- Datenbyte des Status-Registers setzen
		mov INDF,HILFSREG
		mov FSR,#CANS_MSG_DATA1			;Datenbyte in abgehende MSG setzen		(DATA 1)
		mov INDF,HILFSREG
	
		page $E00
		     call WR_EEPROMSR			
		page $200
		
		mov FSR,#CANS_MSG_DBYTE			;Lnge fr MSG setzen 				(DBYTE)
		mov INDF,#$02
			
		page $C00
		     call SendCanMsg 
		page $400
		     jmp parseCanEnd
		;=====================================


WR_EEPROM_DIS	;**** Schreiben des EEPROM sperren (#$F4) ****
		page $E00
		     call WR_DISABLE			
		page $400
	 	     jmp parseCanEnd
		;=====================================


WR_EEPROM_ENA	;**** Schreiben des EEPROM entsperren (#$F5) ****
		page $E00
		     call WR_ENABLE			
		page $400
		     jmp parseCanEnd
		;=====================================


SaveTriState	;**** Auswahl Routine $F6 ****
		page $600
		     call SaveTriToEeprom		    
		page $400 
		     jmp parseCanEnd
		;=====================================		


ReadTriState	;**** Auswahl Routine $F7 ****
		page $600
		     call ReadEepromToTri		    
		page $400 
		     jmp parseCanEnd
		;=========== ENDE DER EEPROM ROUTINEN ============


RequestMotDrive  ;**** Auswahl Routine $E2 ****
		;Nachricht, das Motor immer noch aktiv
		;Sicherungen aktivieren/deaktivieren
		;Zurcksetzen des Counters fr Interrupt-Routine
		mov FSR,#CANE_MSG_DATA1
		mov HILFSREG,INDF
		and HILFSREG,#%01000011		;Maskierung der Daten
		snb INDF.6			;Ist Motor noch aktiv?
		   jmp TestMotAkt

		mov FSR,#VorCountMot		;Zurcksetzten des Vor Counters
		clr INDF

		mov FSR,#StopAktMotC		;Zurcksetzten des Counters
		clr INDF

		mov FSR,#StatusMotor
		clr INDF
		page $400
		   jmp parseCanEnd

TestMotAkt
		mov FSR,#StatusMotor		;Ist Motor bereits aktiv?
		snb INDF.6
		   jmp TestRoutine		;Wenn Motor aktiv Sprung zur Testroutine
		mov RTCC,#$01			;RTCC fr Interrrupt auf 1 setzen	

TestRoutine
		mov FSR,#StatusMotor
		or INDF,HILFSREG		;bernehmen der Daten

		mov FSR,#StopAktMotC		;Zurcksetzen des Counters
		clr INDF

		page $400 
		     jmp parseCanEnd
		;=====================================


PowEndSwitch    ;**** Auswahl Routine $F9 ****
		;Einschalten Spannung Endschalter
		mov FSR,#CANE_MSG_DATA1
		mov HILFSREG,INDF

		snb HILFSREG.0
		    setb Power_EndS

		sb HILFSREG.0
		    clrb Power_EndS

		;Empfangsdaten bernehmen		
		mov StartVar,#CANE_MSG_DBYTE		;Empfangs_Data 0..2		
		mov EndVar,#CANS_MSG_DBYTE		;Sende_Data 0..2
		mov FSR,#CountCopy
		mov INDF,#$03

		page $E00
		     call DataCopy
		page $C00
		     call SendCanMsg 			;Nachricht senden     
		page $400 
		     jmp parseCanEnd
		;=====================================


WEndStufen	;**** Auswahl Routine $FA ****
		;Einstellungen der Microschritte fr Endstufen		
		mov FSR,#CANE_MSG_DATA1
		mov HILFSREG,INDF
		
		mov RE,HILFSREG				;Ausgabe an LED Packs
		setb LED_CS
		nop
		nop
		clrb LED_CS	
			
		mov StartVar,#CANE_MSG_DBYTE		;Empfangs_Data 0..2		
		mov EndVar,#CANS_MSG_DBYTE		;Sende_Data 0..2
		mov FSR,#CountCopy
		mov INDF,#$03

		page $E00
		     call DataCopy			
		page $C00
		     call SendCanMsg 			;Nachricht senden     
		page $400 
		     jmp parseCanEnd
		;===============================


RTriRegister	;**** Auswahl Routine $FB ****
		;Auslesen von 24bit Registern (Geschwindigkeit, Encoder oder Position)
		mov FSR,#CANE_MSG_DATA1			;Empfangs_Data 1		
		mov HILFSREG,INDF
		mov FSR,#Trinamic_Selc			;Trinamic Nummer setzen
		mov INDF,HILFSREG

		mov FSR,#Trinamic_Adr			;Adresse des Trinamic Adressregisters setzen
		mov INDF,#$40

		mov FSR,#Trinamic_Data0			;Zu schreibende Daten fr Trinamic setzen
		mov INDF,#$00
		mov HILFSREG,FSR

		page $600
		     call Para_Write			;Schreiben der Daten aus angegebener Adresse des Trinamic
		page $200

		mov FSR,#CANE_MSG_DATA2			;Empfangs_Data 1		
		mov HILFSREG,INDF
		mov FSR,#Trinamic_Adr			;Trinamic Nummer setzen
		mov INDF,HILFSREG			;(#$48 = Geschwindigkeit), (#$50 = Position), (#$14 = Encoder)

		;Lesen Datenbyte 1
		mov HILFSREG,#Trinamic_Data0		;Zeiger auf Trinamic_Data0
		
		page $600
		     call Para_Read			;Lesen Daten aus angegebener Adresse des Trinamic
		page $200
			
		;Lesen Datanbyte 2
		mov FSR,#Trinamic_Adr			;Adresse des Trinamic Adressregisters setzen
		inc INDF				;(#$49 = Geschwindigkeit), (#$51 = Position), (#$15 = Encoder)
		inc HILFSREG				;Zeiger auf Trinamic_Data1

		page $600
		     call Para_Read			;Lesen Daten aus angegebener Adresse des Trinamic
		page $200
			
		;Lesen Datanbyte 3
		mov FSR,#Trinamic_Adr			;Adresse des Trinamic Adressregisters setzen
		inc INDF				;(#$4A = Geschwindigkeit), (#$52 = Position), (#$16 = Encoder)
		inc HILFSREG				;Zeiger auf Trinamic_Data2

		page $600
		     call Para_Read			;Lesen Daten aus angegebener Adresse des Trinamic
		page $200

		;Empfangsdaten bernehmen		
		mov StartVar,#CANE_MSG_DATA0		;Empfangs_Data 0..2		
		mov EndVar,#CANS_MSG_DATA0		;Sende_Data 0..2
		mov FSR,#CountCopy
		mov INDF,#$03

		page $E00
		     call DataCopy
		page $200

		;Trinamic Daten bernehmen
		mov StartVar,#Trinamic_Data0		;Trinamic_Data 0..2		
		mov EndVar,#CANS_MSG_DATA3		;Sende_Data 2..4
		mov FSR,#CountCopy
		mov INDF,#$03

		page $E00
		     call DataCopy
		page $200

		mov FSR,#CANS_MSG_DBYTE			;Lnge fr MSG setzen 				(DBYTE)
		mov INDF,#$06
								
		page $C00
		     call SendCanMsg 			;Nachricht senden		     
		page $400 
		     jmp parseCanEnd
		;=====================================


Tri_ReadOut	;**** Auswahl Routine $FC ****	
		mov StartVar,#CANE_MSG_DATA1		;Empfangs_Data 1..2		
		mov EndVar,#Trinamic_Selc		;Trinamic_Sel, Trinamic_Adr
		mov FSR,#CountCopy
		mov INDF,#$02

		page $E00
		     call DataCopy
		page $200

		mov StartVar,#CANE_MSG_DATA0		;Empfangs_Data 0..2		
		mov EndVar,#CANS_MSG_DATA0		;Sende_Data 0..2
		mov FSR,#CountCopy
		mov INDF,#$03

		page $E00
		     call DataCopy
		page $200	

		mov HILFSREG,#Trinamic_Data0		;Zeiger auf Datenspeicher ins Hilfsregister laden

		page $600
		     call Para_Read			;Lesen Daten aus angegebener Adresse des Trinamic
		page $200
			
		mov FSR,#Trinamic_Data0
		mov HILFSREG,INDF
		mov FSR,#CANS_MSG_DATA3			;Ausgelesene Daten des Trinamic in MSG setzen 	(DATA 3)
		mov INDF,HILFSREG

		mov FSR,#CANS_MSG_DBYTE			;Lnge fr MSG setzen 				(DBYTE)
		mov INDF,#$04
								
		page $C00
		     call SendCanMsg 
		page $400 
		     jmp parseCanEnd
		;=====================================


Tri_FiFo	;**** Auswahl Routine $FD ****		
		mov FSR,#CANE_MSG_DATA1			;Trinamic Nummer lesen
		mov HILFSREG,INDF					
		mov FSR,#Trinamic_Selc			;Trinamic Nummer setzen
		mov INDF,HILFSREG			

		mov StartVar,#CANE_MSG_DATA2		;Empfangs_Data 2..5		
		mov EndVar,#Trinamic_Data0		;Trinamic_Data0, Trinamic_Data1, Trinamic_Data2, Trinamic_Data3
		mov FSR,#CountCopy
		mov INDF,#$04

		page $E00
		     call DataCopy
		page $200

		mov FSR,#CANS_MSG_DBYTE			;Lnge fr MSG setzen 				(DBYTE)
		mov INDF,#$06
	
		mov StartVar,#CANE_MSG_DATA0		;Empfangs_Data 0..5		
		mov EndVar,#CANS_MSG_DATA0		;Sende_Data 0..5
		mov FSR,#CountCopy
		mov INDF,#$06

		page $E00
		     call DataCopy
		page $600
		     call Write_FiFo			;Routine zum Beschreiben des FiFo							
		page $C00
		     call SendCanMsg  
		page $400 
		     jmp parseCanEnd
		;=====================================





;-Speicherbereich-
org $400
;Definition von Routinen in der LOW Page
L_ReadEepromToTri	jmp ReadEepromToTri
L_SaveTriToEeprom	jmp SaveTriToEeprom
;L_EncBissData		jmp EncBissData


;-Subroutine-
Tri_WriteTo	;**** Auswahl Routine $FE ****
		mov StartVar,#CANE_MSG_DATA1		;Empfangs_Data 1..3		
		mov EndVar,#Trinamic_Selc		;Trinamic_Sel, Trinamic_Adr, Trinamic_Data0
		mov FSR,#CountCopy
		mov INDF,#$03

		page $E00
		     call DataCopy
		page $400

		mov StartVar,#CANE_MSG_DATA0		;Empfangs_Data 0..3		
		mov EndVar,#CANS_MSG_DATA0		;Sende_Data 0..3
		mov FSR,#CountCopy
		mov INDF,#$04

		page $E00
		     call DataCopy
		page $400
		
		mov HILFSREG,#Trinamic_Data0		;Hilfsregister mit Zeiger auf Daten laden

		page $600
		     call Para_Write			;Schreiben der Daten in angegebener Adresse des Trinamic
		page $400

		mov FSR,#CANS_MSG_DBYTE			;Lnge fr MSG setzen 				(DBYTE)
		mov INDF,#$04
								
		page $C00
		     call SendCanMsg 
		page $400
		     jmp parseCanEnd
		;=====================================


;-Subroutine-
RequestReady	;**** Senden der Ready Nachricht $FF ****
		mov FSR,#CANS_MSG_DBYTE
		mov INDF,#$04
		inc FSR
		mov INDF,#$02
		inc FSR
		mov INDF,#CanNodeKennung
		inc FSR
		mov INDF,#$00
		inc FSR
		mov INDF,#$00

		page $C00
		     call SendCanMsg
		page $400 
		     jmp parseCanEnd			
		;======================


;-Subroutine-
RequestReadPort	;**** Senden des Status aller Ports $00 ****	
		mov FSR,#CANS_MSG_DBYTE
		mov INDF,#$04

		inc FSR
		mov INDF,#$22

		inc FSR
		mov INDF,RA

		inc FSR
		mov INDF,RB
						
		inc FSR
		mov INDF,RC

		inc FSR
		mov INDF,RD

		inc FSR
		mov INDF,RE

		page $C00
		     call SendCanMsg
	    	page $400 
		     jmp parseCanEnd
		;======================


;Subroutine-
BissAbsEncoder	;**** Routine fr Lesen der Encoder $E0 ****
		;Informationen ueber Anzahl der Datenworte
		mov FSR,#CANE_MSG_DBYTE
		mov HILFSREG,INDF
	
		cjb HILFSREG,#$04,parseCanEnd		;berprfung ob Anzahl korrekt

		mov FSR,#CANS_MSG_DBYTE			;Anzahl der zu sendenden Daten
		mov INDF,HILFSREG
		sub HILFSREG,#$03
		mov FSR,#MB3z_DatCount			;Laden der Informationen fuer Anzahl der zu sendenden/lesenden Daten
		mov INDF,HILFSREG

		mov FSR,#CANE_MSG_DATA1
		mov HILFSREG,INDF
		mov FSR,#CANS_MSG_DATA1
		mov INDF,HILFSREG
		mov FSR,#MB3z_ICSelc			;Laden der Daten fuer Auswahl des Biss Interfaces
		mov INDF,HILFSREG
		and INDF,#%00000011			;Maskierung der Auswahl der Biss Interfaces

		RR HILFSREG				;Verschieben der Bits (7->6)
		RR HILFSREG				;Verschieben der Bits (6->5)
		RR HILFSREG				;Verschieben der Bits (5->4)
		RR HILFSREG				;Verschieben der Bits (4->3)
		
		;Informationen ueber B7(jetzt B3), B6(jetzt B2), B5(jetzt B1), B4(jetzt B0)
		mov FSR,#MB3z_Befehl			;Laden der Informationen fuer Biss Befehl
		mov INDF,HILFSREG
		and INDF,#%00001111			;Maskierung des Befehls fuer Biss Interface

		mov StartVar,#CANE_MSG_DATA2		;Empfangs_Data 2..		
		mov EndVar,#MB3z_Adr			;Informationen_Data 0..
		mov FSR,#CANS_MSG_DBYTE
		mov HILFSREG,INDF
		mov FSR,#CountCopy
		mov INDF,HILFSREG
		sub INDF,#$02

		page $E00
		     call DataCopy
		page $800
		     call EncBissData
		page $400

		mov FSR,#CANS_MSG_DATA0
		mov INDF,#$E0

		mov StartVar,#MB3z_Adr			;Adresse		
		mov EndVar,#CANS_MSG_DATA2		;zu sendende_Data 2..
		mov FSR,#CANS_MSG_DBYTE			;Anzahl der zu sendenden Daten
		mov HILFSREG,INDF
		mov FSR,#CountCopy
		mov INDF,HILFSREG
		sub INDF,#$02

		page $E00
		     call DataCopy
		page $C00
		     call SendCanMsg
	    	page $400 
		     jmp parseCanEnd
		;======================



SetEndVorCM	;**** Routine fr Setzes des Zeitfeldes fr Totmann $E3 ****
		mov FSR,#CANE_MSG_DATA1
		mov HILFSREG,INDF
		and HILFSREG,#%00000111
		or HILFSREG,#%00000010
		mov FSR,#EndVorCM
		mov INDF,HILFSREG

		;Empfangsdaten bernehmen		
		mov StartVar,#CANE_MSG_DBYTE		;Empfangs_Data 0..2		
		mov EndVar,#CANS_MSG_DBYTE		;Sende_Data 0..2
		mov FSR,#CountCopy
		mov INDF,#$03

		page $E00
		     call DataCopy
		page $C00
		     call SendCanMsg 			;Nachricht senden     
		page $400 
		     jmp parseCanEnd
		;=====================================


BissPorts	;**** Routine fr Lesen der Encoder $E1 ****
		mov StartVar,#CANE_MSG_DATA0		;Empfangs_Data 0..1		
		mov EndVar,#CANS_MSG_DATA0		;zu sendende Daten 0..1
		mov FSR,#CountCopy
		mov INDF,#$02
		page $E00
		     call DataCopy
		page $400

		mov FSR,#CANE_MSG_DATA1
		sb INDF.0
		   clrb RB.6				;Zurcksetzen von GetSENS (Test off)
		snb INDF.0
		   setb RB.6				;Setzen von GetSENS (Test on)
		sb INDF.1
		   clrb RB.7				;Zurcksetzen von NRES
		snb INDF.1
		   setb RB.7				;Setzen von NRES

		snb INDF.2				;Einmaliges aufrufen von GetSens
	          setb RB.6
		nop
		nop
		snb INDF.2				;zurcksetzen nach Aufruf von GetSens
		  clrb RB.6
		  
		mov FSR,#CANS_MSG_DATA2			;Informationen_Data 0..
		mov INDF,RA
		and INDF,#%00110000			;Maskieren der relevanten Informationen
	
		mov FSR,#CANS_MSG_DATA3			;Informationen_Data 0..
		mov INDF,RB
		and INDF,#%00001100			;Maskieren der relevanten Informationen

		mov FSR,#CANS_MSG_DBYTE		;Anzahl der zu sendenden Daten
		mov INDF,#$04

		page $C00
		     call SendCanMsg 			;Nachricht senden     
		page $400 
		     jmp parseCanEnd
		;=====================================
	
;Auswertung der Nachricht und Aktivierung der automatischen Sensordatenabfrage
Sensor_Data
		mov FSR,#CANS_MSG_DBYTE			;Anzahl Datenbytes
		mov INDF,#$03
		mov FSR,#CANS_MSG_DATA0
		mov INDF,#$E5
	
		mov FSR,#CANE_MSG_DATA1			;Informationen_Data 1 auslesen
		mov HILFSREG,INDF
		mov FSR,#StatusSensorDat		;Statusbyte setzen
		mov INDF,HILFSREG
		
		;-Auswahl welche IC aktiviert werden sollen
		;Deaktivieren AGS
		cje INDF,#$00,Deakt_AGS		;Deaktivieren des AGS Mode fr IC1 und IC2

		;Aktivieren AGS
		cje INDF,#$40,Start_IC1		;Starten des AGS Mode fr IC1
		cje INDF,#$42,Start_IC1		;Starten des AGS Mode fr IC1
		cje INDF,#$41,Start_IC2		;Starten des AGS Mode fr IC2
		    jmp parseCanEnd		;Beenden der Routine

		;------------------------
Start_IC1
		mov FSR,#MB3z_ICSelc		;Setzen von MB3z_ICSelc
		mov INDF,#$00
		page $800
		     call L_AGSStart
		page $400

		;berprfen ob 2. Baustein auch aktiviert werden soll
		mov FSR,#StatusSensorDat
		cjne INDF,#$42,parseCanEnd
		;------------------------
Start_IC2
		mov FSR,#MB3z_ICSelc		;Setzen von MB3z_ICSelc
		mov INDF,#$01
		page $800
		     call L_AGSStart
		page $400
		     jmp parseCanEnd
		;=====================================

		;Deaktivieren des AGS Mode
Deakt_AGS
		mov FSR,#MB3z_ICSelc		;Setzen von MB3z_ICSelc
		mov INDF,#$00
		mov FSR,#CANS_MSG_DATA1		;Send Data1 auf IC1 setzen
		mov INDF,#$00		
Deak_IC2
		page $800
  		     call READSENS		;Auslesen der Sensordaten
		page $400

		;berprfung Fehler SENS Data
		mov FSR,#Fehler_Count
		cje INDF,#$00,SEND_DEAK      	;Fehler Sensordaten senden
		;-----------------------
		
		;bernehmen der neuen Daten in Speicher 
		mov StartVar,#SensPosD0		;Informationen_Data 0..
		mov EndVar,#CANS_MSG_DATA2
		mov FSR,#CountCopy
		mov INDF,#$02
		page $E00
		     call DataCopy
		page $400
		mov FSR,#CANS_MSG_DBYTE		;Anzahl Datenbytes auf 4 setzen
		mov INDF,#$04
SEND_DEAK		
		page $C00
		     call SendCanMsg 		;Nachricht senden     
		page $400 
		;-----------------------

		;-Deaktiviern AGS-
		;Setzen der Anzahl zu bearbeitender Daten
		mov FSR,#MB3z_DatCount		;Anzahl der auszulesenden Daten
		mov INDF,#$01
		mov FSR,#MB3z_Befehl		;Laden des Write Befehls
		mov INDF,#Ins_BissWriteData
		INC FSR				;Laden der Adresse
		mov INDF,#MB3_BefehlsReg	;Adresse F4
		INC FSR				;Setzen des Inhaltes auf (AGS)
		mov INDF,#$00
		page $800
		    call EncBissData		;Aufrufen der Routine zum Zugriff auf IC_MB3
		page $400
		;-----------------------

		mov FSR,#MB3z_ICSelc		;Setzen von MB3z_ICSelc
		cje INDF,#$01,parseCanEnd
		mov INDF,#$01

		mov FSR,#CANS_MSG_DATA1		;Send Data1 auf IC2 setzen
		mov INDF,#$01

		    jmp Deak_IC2

		;=====================================
parseCanEnd		
		mov FSR,#MCP2515_CanFlags
		clrb INDF.0
ret
;===============================================================





