;==========================================================================================
; Programm mit Hilfsfunktionen fr SPI_INTERFACE
;
; Visitec GmbH Januar 2008 by N.Fritsch   	     
; 
; SPI_UNIVERSAL.INC		(Speicherbereich $C50 - $D5B)
;==========================================================================================
;
; DEFINIERTE ROUTINEN SIND:
;;	SUBROUTINEN fr SPI Kommunikation
;	=================================
;
;	sendbyte		-	sendet ein Byte seriell ber den Serial Data Out Pin des SX - 
;					erwartet die Adresse des Bytes in  FSR (File Select Register)
;					Ist sowohl fr High als auch fr Low Flanke verwendbar
;
;	getbyte			-	empfngt ein Byte seriell ber den Serial Data In Pin des SX   -   Adresse,
;					an die das Byte geschrieben werden soll in FSR
;					ACHTUNG! Datenbernahme erfolgt bei High Flanke
;	
; Verwendung im Hauptprogramm: 	include "SPI_UNIVERSAL.INC"
;
; Bemerkungen:			Subroutinen sind im Programmspeicher ab Page $800 abgelegt
;				zum Zugriff auf diese Routinen ist nach dem Aufruf im Hauptprogramm 
;				wieder die entsprechende Adresse zu setzen da die ersten 3 Bits (7,6,5) 
;				des Statusregisters auf einen anderen Speicherbereich zeigen
;				des Statusregisters anzupassen in diesem Fall Page0 ==> page$000
;
;				Bsp.: page $800
;				      	    call sendbyte
;				      page $000
;
;
;				Achtung!!! Clock darf 50 Mhz nicht bersteigen, ansonsten muss ein weiterer nop eingefgt werden
;
; Speicherbereich($C50-$D5B)
; Lnge ($10C = 268) Speicherwrter
;==========================================================================================================

;---------------Steuerleitungen des SX fr MCP2510-----------
SDO_SX				equ	RC.2			;Serial Data aus dem SX in Baustein
SDI_SX				equ	RC.1			;Serial Data in den SX aus Baustein
SCK_SX				equ	RC.0			;Serial Clock Pin

;------Macro fr 4 Cycle Zeitverzgerung-----
wait_4Cycle	macro
		nop
		nop
		nop
		nop
		nop
endm
;---------------------------------

wait_2Cycle	macro
		nop
		nop
		nop
endm
;================ allgemeine Befehle zum Senden und Empfangen eines Bytes =================================
;--Speicherbereich fr Routinen--
org $C50

;-Subroutine zum Senden- 			sendet das Byte, auf das der Zeiger FSR zeigt seriell
sendbyte	
		movb SDO_SX,INDF.7		;Daten BIT 7
		wait_2Cycle			
		clrb SCK_SX				;Clock LOW
		wait_4Cycle				;Verzgerung mind. 45ns		
		setb SCK_SX				;Clock HIGH
		wait_2Cycle					;mind. 10ns Verzgerung

		movb SDO_SX,INDF.6		;Daten BIT 6
		wait_2Cycle
		clrb SCK_SX				;Clock LOW
		wait_4Cycle				;Verzgerung mind. 45ns				
		setb SCK_SX				;Clock HIGH
		wait_2Cycle					;mind. 10ns Verzgerung

		movb SDO_SX,INDF.5		;Daten BIT 5
		wait_2Cycle			
		clrb SCK_SX				;Clock LOW
		wait_4Cycle				;Verzgerung mind. 45ns		
		setb SCK_SX				;Clock HIGH
		wait_2Cycle					;mind. 10ns Verzgerung

		movb SDO_SX,INDF.4		;Daten BIT 4
		wait_2Cycle				
		clrb SCK_SX				;Clock LOW
		wait_4Cycle				;Verzgerung mind. 45ns		
		setb SCK_SX				;Clock HIGH
		wait_2Cycle					;mind. 10ns Verzgerung

		movb SDO_SX,INDF.3		;Daten BIT 3
		wait_2Cycle
		clrb SCK_SX				;Clock LOW
		wait_4Cycle				;Verzgerung mind. 45ns		
		setb SCK_SX				;Clock HIGH
		wait_2Cycle					;mind. 10ns Verzgerung

		movb SDO_SX,INDF.2		;Daten BIT 2
		wait_2Cycle
		clrb SCK_SX				;Clock LOW
		wait_4Cycle				;Verzgerung mind. 45ns		
		setb SCK_SX				;Clock HIGH
		wait_2Cycle					;mind. 10ns Verzgerung

		movb SDO_SX,INDF.1		;Daten BIT 1
		wait_2Cycle
		clrb SCK_SX				;Clock LOW
		wait_4Cycle				;Verzgerung mind. 45ns		
		setb SCK_SX				;Clock HIGH
		wait_2Cycle					;mind. 10ns Verzgerung
	
		movb SDO_SX,INDF.0		;Daten BIT 0
		wait_2Cycle
		clrb SCK_SX				;Clock LOW
		wait_4Cycle				;Verzgerung mind. 45ns		
		setb SCK_SX				;Clock HIGH
ret
;-------------------------------------


;-Subroutine zum Empfangen-			liest Byte seriell vom MCP und bergibt in Speicherzelle, auf die FSR zeigt
getbyte		
		clrb SCK_SX				;Clock LOW
		wait_4Cycle				;Verzgerung mind. 45ns					
		setb SCK_SX				;Clock HIGH
		wait_2Cycle					;mind. 10ns Verzgerung
		movb INDF.7,SDI_SX		;Daten Bit7		
		wait_2Cycle					;mind. 10ns Verzgerung

				
		clrb SCK_SX				;Clock LOW
		wait_4Cycle				;Verzgerung mind. 45ns					
		setb SCK_SX				;Clock HIGH
		wait_2Cycle					;mind. 10ns Verzgerung
		movb INDF.6,SDI_SX		;Daten Bit6		
		wait_2Cycle					;mind. 10ns Verzgerung
					
		clrb SCK_SX				;Clock LOW
		wait_4Cycle				;Verzgerung mind. 45ns					
		setb SCK_SX				;Clock HIGH
		wait_2Cycle					;mind. 10ns Verzgerung
		movb INDF.5,SDI_SX		;Daten Bit5		
		wait_2Cycle			
	
		clrb SCK_SX				;Clock LOW
		wait_4Cycle				;Verzgerung mind. 45ns					
		setb SCK_SX				;Clock HIGH
		wait_2Cycle					;mind. 10ns Verzgerung
		movb INDF.4,SDI_SX		;Daten Bit4		
		wait_2Cycle	
		
		clrb SCK_SX				;Clock LOW
		wait_4Cycle				;Verzgerung mind. 45ns			
		setb SCK_SX				;Clock HIGH
		wait_2Cycle					;mind. 10ns Verzgerung
		movb INDF.3,SDI_SX		;Daten Bit3		
		wait_2Cycle	
		
		clrb SCK_SX				;Clock LOW
		wait_4Cycle				;Verzgerung mind. 45ns	
		setb SCK_SX				;Clock HIGH
		wait_2Cycle					;mind. 10ns Verzgerung
		movb INDF.2,SDI_SX		;Daten Bit2		
		wait_2Cycle	

		clrb SCK_SX				;Clock LOW
		wait_4Cycle				;Verzgerung mind. 45ns					
		setb SCK_SX				;Clock HIGH
		wait_2Cycle					;mind. 10ns Verzgerung
		movb INDF.1,SDI_SX		;Daten Bit1		
		wait_2Cycle	

		clrb SCK_SX				;Clock LOW
		wait_4Cycle				;Verzgerung mind. 45ns					
		setb SCK_SX				;Clock HIGH
		wait_2Cycle					;mind. 10ns Verzgerung
		movb INDF.0,SDI_SX		;Daten Bit0			
ret

;Programmcounter bei (D5B)
;-------------------------------------