;==========================================================================================
; Programm mit Subroutinen fr die Trinamics
;
; Visitec GmbH Februar 2008 by N.Fritsch   	     
; 
; Trinamics_V5.inc
;==========================================================================================
;
; DEFINIERTE ROUTINEN SIND:
;
;	Tri_Init:	Macro zur Initialisierung der Trinamics. Diese werden definiert zurckgesetzt.
;
;	Tri_Data_IN:	Datenport wird von SX Seite als Ausgang gelegt ==> Daten in Trinamics
;
;	Tri_Data_OUT:	Datenport wird von SX Seite als Eingang gelegt ==> Daten aus Trinamics
;
;	Para_Write:	Routine zur Auswahl des entsprechendne Trinamic, in den geschrieben werden soll.
;			Notwendige Daten die vor Aufruf dieser Routine gesetzt sein mssen sind:
;						- Auswahl des Trinamic (Trinamic_Selc) (0=1; 1=2; 2= beide)
;						- Adresse fr Register des Trinamics (Trinamic_Adr)
;						- Daten zum Schreiben in ausgewhlte Adresse (Trinamic_Data)
;
;	Para_Read:	Routine zur Auswahl des entsprechendne Trinamic, aus den gelesen werden soll.
;			Notwendige Daten die vor Aufruf dieser Routine gesetzt sein mssen sind:
;						- Auswahl des Trinamic (Trinamic_Selc) (0=1; 1=2; 2= beide)
;						- Adresse fr Register des Trinamics (Trinamic_Adr)
;	
;	StatusMotor:	Bit0	Trinamic 0 ist ausgewhlt (0)/ Trinamic 1 ist ausgewhlt (1)
;			Bit1	beide Trinamics sind ausgewhlt (1)
;			Bit2    ---
;			Bit3    ---
;			Bit4    --- 
;			Bit5    ---
;			Bit6    Motor od. Motoren sind aktiv (1)
;			Bit7    NotStop soll ausgefhrt werden (1)
;
; Verwendung im Hauptprogramm: 	include "Trinamics.inc"
;
; Bemerkungen:	----		
;
;-------------------------------------------------------------------------------------------
;		p a r a l l e l   K o m u n i k a t i o n   m i t   T r i n a m i c s
;===========================================================================================

;-Steuerleitungen des SX fr Trinamics-
Tri_NRes0	equ RD.0
Tri_NRes1	equ RD.7
Tri_ALE		equ RD.1
Tri_NCS0	equ RD.3
Tri_NCS1	equ RD.6
Tri_NWE		equ RD.4
Tri_NOE		equ RD.5
Tri_Data	equ RE

;-Konstante fr Trinamics Kommunikation-
REData_OUT	equ	%00000000		;...Port E => Datenleitungen Trinamics (Ausgang SX)
REData_IN	equ	%11111111		;...Port E => Datenleitungen Trinamics (Eingang SX)

;-Variablendefinition fr Trinamics-
Trinamic_Selc	equ	$40
Trinamic_Adr	equ	$41
Trinamic_Data0	equ	$42
Trinamic_Data1	equ	$43
Trinamic_Data2	equ	$44
Trinamic_Data3	equ	$45
Trinamic_Data4	equ	$46
Trinamic_Data5	equ	$47
Trinamic_Count	equ	$48

;Variablendefinition fuer Sicherheitsroutine
StopAktMotC	equ	$49
VorCountMot	equ	$4A
StatusMotor	equ	$4B
EndVorCM	equ	$4C


;-------------------------------------------------------------------------------------------
;===========================================================================================

;-Macros-
;----------- Macro zur Initialisierung der Trinamics ----------
Tri_Init	macro
		;Reset der Trinamics
		clrb Tri_NRes0
		nop
		nop
		clrb Tri_NRes1

		mov FSR,#COUNTER0
		mov INDF,#$FF
		
		page $E00
		     call TimeEater2
		page $600

		setb Tri_NRes0
		nop
		nop
		setb Tri_NRes1
endm
;---------------------------------------------------------------


;-------------- Datenrichtung auf Eingang in SX ----------------
Tri_Data_IN	macro
		CLR RE
		mode $0F	
		mov !RE,#REData_IN
endm
;---------------------------------------------------------------


;-------------- Datenrichtung auf Ausgang in SX ----------------
Tri_Data_OUT	macro
		mode $0F	
		mov !RE,#REData_OUT
		CLR RE
endm
;---------------------------------------------------------------

;===================================================================================
;-Speicheradresse fr Programmcode-
org $600  

;Sprungmarken fr Subroutinen im unteren Bereich
L_READ_POS	jmp READ_POS
L_SAVE_POS	jmp SAVE_POS

;-Subroutinen-
;- Routine zum parallelen Schreiben in entsprechenden Trinamic -
;---- Schreibt Byte in entsprechende Adresse des Trinamic 0 ----
Para_Write	
		mov FSR,#Trinamic_Adr	
		mov Tri_Data,INDF		;Adresse setzen
		
		setb Tri_ALE
		mov FSR,#Trinamic_Selc
		clrb Tri_NWE
		csne INDF,#$00			;berprfen ob Trinamic 0 beschrieben werden soll
		    clrb Tri_NCS0			;aktivieren Trinamic 0
		
		csne INDF,#$01			;berprfen ob Trinamic 1 beschrieben werden soll
		    clrb Tri_NCS1			;aktivieren Trinamic 1
		
		csne INDF,#$02			;berprfen ob beide Trinamics beschrieben werden sollen	
    		    clrb Tri_NCS0			;aktivieren Trinamic 0
		csne INDF,#$02
    		    clrb Tri_NCS1			;aktivieren Trinamic 1
		nop				;Zeitverzgerung
		clrb Tri_ALE
		
		mov FSR,HILFSREG
		mov Tri_Data,INDF		;Daten schreiben
		
		nop
		nop

		or RD,#%01001000		;Deaktivieren aller Trinamics (RD.6, RD.3)
		nop				;Zeitverzgerung
		setb Tri_NWE	
ret
;---------------------------------------------------------------

;-Subroutinen-
;--- Routine zum parallelen Lesen in entsprechenden Trinamic ---
;-- Lesen des Bytes aus entsprechende Adresse des Trinamic 0 ---
Para_Read
		Tri_Data_OUT			;Datenleitungen (RE) in SX auf Ausgang setzten

		mov FSR,#Trinamic_Adr	
		mov Tri_Data,INDF		;Adresse setzen

		setb Tri_ALE
		mov FSR,#Trinamic_Selc		;Trinamicauswahl einlesen
		setb Tri_NWE
		csne INDF,#$00			;berprfen ob Trinamic 0 gelesen werden soll
		    clrb Tri_NCS0			;aktivieren Trinamic 0

		csne INDF,#$01			;berprfen ob Trinamic 1 gelesen werden soll
		    clrb Tri_NCS1			;aktivieren Trinamic 1
		nop				;Zeitverzgerung
		clrb Tri_ALE

		Tri_Data_IN			;Datenleitungen (RE) in SX auf Eingang setzten

		nop
		nop
	
		clrb Tri_NOE
	
		nop
		nop
		nop
		nop
		
		mov FSR,HILFSREG
		mov INDF,Tri_Data		;Daten einlesen	
	
		nop
		nop
	
		or RD,#%01001000		;Deaktivieren aller Trinamics (RD.6, RD.3)
		nop				;Zeitverzgerung
		setb Tri_NOE
		Tri_Data_OUT			;Datenleitungen (RE) in SX auf Ausgang setzten
ret
;---------------------------------------------------------------

;-Subroutine-
Write_FiFo
		;-ersten Byte-
		mov FSR,#Trinamic_Adr
		mov INDF,#$00				;Adress-Register fr abgehende MSG setzen
		mov HILFSREG,#Trinamic_Data0		;Zeiger auf Daten ins Hilfsregister legen
	
		     call Para_Write			;Schreiben der Daten in angegebener Adresse des Trinamic

		;-zweites Byte-
		mov FSR,#Trinamic_Adr
		inc INDF				;Adresse fr Trinamic um 1 erhhen		
		inc HILFSREG				;Zeiger auf Trinamic_Data1

		     call Para_Write			;Schreiben der Daten in angegebener Adresse des Trinamic

		;-drittes Byte-
		mov FSR,#Trinamic_Adr
		inc INDF				;Adresse fr Trinamic um 1 erhhen		
		inc HILFSREG				;Zeiger auf Trinamic_Data2

		     call Para_Write			;Schreiben der Daten in angegebener Adresse des Trinamic

		;-viertes Byte-
		mov FSR,#Trinamic_Adr
		inc INDF				;Adresse fr Trinamic um 1 erhhen
		inc HILFSREG				;Zeiger auf Trinamic_Data3		

		     call Para_Write			;Schreiben der Daten in angegebener Adresse des Trinamic

ret

;---------------------------------------------------------------
;-Subroutine
Akt_NotStop		
		mov !option,#%11000000		;Deaktivieren der Interrupt Routine

		mov RTCC,#$00			;Zurcksetzen des RTCC Registers
		mov FSR,#StatusMotor
		mov HILFSREG,INDF
		clr INDF
		and HILFSREG,#$03			;Maskierung, das nur noch Trinamicauswahl zu sehen

		mov FSR,#VorCountMot			;Zurcksetzten des Vor Counters
		clr INDF

		mov FSR,#Trinamic_Selc			;bernahme der Daten in Trinamic
		mov INDF,HILFSREG

		mov StartVar,#$00
		mov EndVar,#Trinamic_Data0		;Trinamic_Adr, Trinamic_Data0, Trinamic_Data1, Trinamic_Data2		
		mov FSR,#CountCopy
		mov INDF,#$04

		page $E00
		     call Tab_Read			;Auslesen der Daten aus der Tabelle
		page $600
	 	     call Write_FiFo
		
		mov FSR,#CANS_MSG_DBYTE			;Lnge fr MSG setzen 				(DBYTE)
		mov INDF,#$06
		INC FSR
		mov INDF,#$E2				;Datenbyte 0

		mov StartVar,#Trinamic_Selc		;Trinamic_Sel, Trinamic_Adr, Trinamic_Data0, Trinamic_Data1, Trinamic_Data2		
		mov EndVar,#CANS_MSG_DATA1		;Sende_Data 1..5
		mov FSR,#CountCopy
		mov INDF,#$05	

		page $E00
		     call DataCopy
		page $C00
		     call SendCanMsg 
		page $600 
ret	

;---------------------------------------------------------------

;-Routine zum Lesen des EEPROM in Trinamic
ReadEepromToTri
		mov FSR,#Trinamic_Selc			;Auswahl des Trinamics
		mov INDF,#$00

		mov FSR,#EEPROM_ADR			;Auswahl des Speicherbereiches EEPROM
		mov INDF,#$10

	     	    call L_READ_POS
		;--------------------------------		

		mov FSR,#Trinamic_Selc			;Auswahl des Trinamics
		mov INDF,#$01

		mov FSR,#EEPROM_ADR			;Auswahl des Speicherbereiches EEPROM
		mov INDF,#$20

	            call L_READ_POS
ret
;===============================================================

;-Routine zum Schreiben des EEPROM aus Trinamicdaten
SaveTriToEeprom
		;Lesen der Position fr Trinamic 0
		mov FSR,#Trinamic_Selc			;Auswahl beider Trinamics
		mov INDF,#$02
		mov FSR,#Trinamic_Adr			;write fifo command
		mov INDF,#$40
		mov FSR,#Trinamic_Data0
		mov INDF,#$00
		mov HILFSREG,FSR

	             call Para_Write

		;-Ablegen der Daten aus Trinamic 1 in EEPROM-
		mov FSR,#Trinamic_Selc		;Auswahl des Trinamics
		mov INDF,#$00

		mov FSR,#EEPROM_ADR		;Auswahl des Speicherbereiches EEPROM
		mov INDF,#$10				

	            call L_SAVE_POS
		;--------------------------------------------

		page $E00
	            call WaitToWriteEeprom
		page $600
		
		;-Ablegen der Daten aus Trinamic 2 in EEPROM-
		mov FSR,#Trinamic_Selc		;Auswahl des Trinamics
		mov INDF,#$01

		mov FSR,#EEPROM_ADR		;Auswahl des Speicherbereiches EEPROM
		mov INDF,#$20				

	    	    call L_SAVE_POS
		;-------------------------------------------
ret
;===============================================================
	
SAVE_POS
		;Achtung Trinamic muss ausgewhlt sein (Trinamic_Selc)
		mov FSR,#Trinamic_Adr				
		mov INDF,#$50				;read position tri0 
		mov FSR,#Trinamic_Count
		mov INDF,#$03
				
		mov HILFSREG,#EEPROM_DATA0		;Daten in EEPROM Variablen speichern
		
REP_READ
	     	     call Para_Read			;Lesen der Daten
	
		inc HILFSREG				;Erhhen des Bufferzhlers
	
		mov FSR,#Trinamic_Adr			;Adresse um 1 erhhen	
		inc INDF				

		mov FSR,#Trinamic_Count			;Counter fr Anzahl der auszulesenden Daten prfen
		decsz INDF
	    	    jmp REP_READ			;Sprung falls noch mehr Daten in EEPROM zu legen 

		mov FSR,#Trinamic_Adr				
		mov INDF,#$92				;read motor power tri0 

	     	     call Para_Read			;Lesen der Daten
			
		;EEPROM fr Schreibzugriff aktivieren
		page $E00
		     call WR_ENABLE			
		page $600

		;Ablegen der Daten in EEPROM
		mov FSR,#EEPROM_INSTR			;Instruction Register laden (Auswahl des Speicherbereiches)
		mov INDF,#EEPROM_WRFL			;(oberer Speicherbereich)
		
		mov FSR,#EEPROM_COUNT			;Anzahl der zu schreibenden Bytes
		mov INDF,#$04

		page $E00
	     	     call WR_EEPROMSP			;Schreiben in EEPROM
		page $600


ret
;===============================================================


READ_POS
		;-Lesen der Daten fr Trinamic 0-
		mov FSR,#EEPROM_INSTR			;set Fifo Command
		mov INDF,#EEPROM_RDFL			;Lese Befehl
							
		mov FSR,#EEPROM_COUNT
		mov INDF,#$04
		mov FSR,#Trinamic_Count
		mov INDF,#$03

		page $E00
		     call RD_EEPROMSP			;Lesen der Daten aus EEPROM
		page $600
			
		;-Schreiben in Trinamic-
		mov FSR,#Trinamic_Adr			;write fifo command
		mov INDF,#$00
	
		mov FSR,#Trinamic_Data0
		mov INDF,#%01001101
		mov HILFSREG,FSR

		     call Para_Write			;Schreiben in Trinamic 0

		mov HILFSREG,#EEPROM_DATA0
		mov FSR,#Trinamic_Adr			;write fifo command
		inc INDF

REP_W2Tri
		     call Para_Write			;Schreiben in Trinamic 0

		mov FSR,#Trinamic_Adr			;Adresse um 1 erhhen
		inc INDF

		inc HILFSREG				;Erhhen des Bufferzhlers
		
		mov FSR,#Trinamic_Count
		decsz INDF
		    jmp REP_W2Tri

		mov FSR,#Trinamic_Adr
		mov INDF,#$92				;write motor power
	
		mov HILFSREG,#EEPROM_DATA3
	
		     call Para_Write			;Schreiben in Trinamic 0
ret
;===============================================================


;---------------------------------------------------------------

